@@ -1,7 +1,7 @@
 import datetime
-import errno
 import os
 import itertools
+import re
 import tempfile
 import textwrap
 import logging
@@ -21,10 +21,10 @@ import features.cloud as cloud
 
 from features.util import emit_spinner_on_travis, lxc_get_property, build_debs
 
-ALL_SUPPORTED_SERIES = ["bionic", "focal", "trusty", "xenial"]
+ALL_SUPPORTED_SERIES = ["bionic", "focal", "xenial"]
 
-DAILY_PPA = "http://ppa.launchpad.net/canonical-server/ua-client-daily/ubuntu"
-DAILY_PPA_KEYID = "8A295C4FB8B190B7"
+DAILY_PPA = "http://ppa.launchpad.net/ua-client/daily/ubuntu"
+DAILY_PPA_KEYID = "6E34E7116C0BC933"
 
 USERDATA_BLOCK_AUTO_ATTACH_IMG = """\
 #cloud-config
@@ -34,12 +34,19 @@ bootcmd:
  - chmod 755 /usr/bin/ua
 """
 
-USERDATA_APT_SOURCE_PPA_TRUSTY = """\
-apt_sources:  # for trusty
-  - source: deb {ppa_url} $RELEASE main
-    keyid: {ppa_keyid}
-packages: [{packages}]
-"""
+# we can't use write_files because it will clash with the
+# lxd vendor-data write_files that is necessary to set up
+# the lxd_agent on some releases
+USERDATA_RUNCMD_PIN_PPA = """\
+runcmd:
+  - "printf \\"Package: *\\nPin: release o=LP-PPA-ua-client-daily\\nPin-Priority: 1001\\n\\" > /etc/apt/preferences.d/uaclient"
+"""  # noqa: E501
+
+USERDATA_RUNCMD_ENABLE_PROPOSED = """
+runcmd:
+  - printf \"deb http://archive.ubuntu.com/ubuntu/ {series}-proposed restricted main multiverse universe\" > /etc/apt/sources.list.d/uaclient-proposed.list
+  - apt-get update
+"""  # noqa: E501
 
 USERDATA_APT_SOURCE_PPA = """\
 apt:
@@ -47,7 +54,6 @@ apt:
     ua-tools-ppa:
         source: deb {ppa_url} $RELEASE main
         keyid: {ppa_keyid}
-packages: [{packages}]
 """
 
 PROCESS_LOG_TMPL = """\
@@ -109,7 +115,13 @@ class UAClientBehaveConfig:
     # These variables are used in .from_environ() to convert the string
     # environment variable input to the appropriate Python types for use within
     # the test framework
-    boolean_options = ["build_pr", "image_clean", "destroy_instances"]
+    boolean_options = [
+        "build_pr",
+        "image_clean",
+        "destroy_instances",
+        "cache_source",
+        "enable_proposed",
+    ]
     str_options = [
         "aws_access_key_id",
         "aws_secret_access_key",
@@ -121,6 +133,7 @@ class UAClientBehaveConfig:
         "gcp_project",
         "contract_token",
         "contract_token_staging",
+        "contract_token_staging_expired",
         "machine_type",
         "private_key_file",
         "private_key_name",
@@ -129,6 +142,8 @@ class UAClientBehaveConfig:
         "artifact_dir",
         "ppa",
         "ppa_keyid",
+        "userdata_file",
+        "check_version",
     ]
     redact_options = [
         "aws_access_key_id",
@@ -139,6 +154,7 @@ class UAClientBehaveConfig:
         "az_subscription_id",
         "contract_token",
         "contract_token_staging",
+        "contract_token_staging_expired",
     ]
 
     # This variable is used in .from_environ() but also to emit the "Config
@@ -161,16 +177,21 @@ class UAClientBehaveConfig:
         build_pr: bool = False,
         image_clean: bool = True,
         destroy_instances: bool = True,
+        cache_source: bool = True,
+        enable_proposed: bool = False,
         machine_type: str = "lxd.container",
         private_key_file: str = None,
         private_key_name: str = "uaclient-integration",
         reuse_image: str = None,
         contract_token: str = None,
         contract_token_staging: str = None,
+        contract_token_staging_expired: str = None,
         debs_path: str = None,
         artifact_dir: str = None,
         ppa: str = DAILY_PPA,
         ppa_keyid: str = DAILY_PPA_KEYID,
+        userdata_file: str = None,
+        check_version: str = None,
         cmdline_tags: "List" = []
     ) -> None:
         # First, store the values we've detected
@@ -183,8 +204,11 @@ class UAClientBehaveConfig:
         self.gcp_credentials_path = gcp_credentials_path
         self.gcp_project = gcp_project
         self.build_pr = build_pr
+        self.cache_source = cache_source
+        self.enable_proposed = enable_proposed
         self.contract_token = contract_token
         self.contract_token_staging = contract_token_staging
+        self.contract_token_staging_expired = contract_token_staging_expired
         self.image_clean = image_clean
         self.destroy_instances = destroy_instances
         self.machine_type = machine_type
@@ -196,6 +220,8 @@ class UAClientBehaveConfig:
         self.artifact_dir = artifact_dir
         self.ppa = ppa
         self.ppa_keyid = ppa_keyid
+        self.userdata_file = userdata_file
+        self.check_version = check_version
         self.filter_series = set(
             [
                 tag.split(".")[1]
@@ -219,6 +245,7 @@ class UAClientBehaveConfig:
             ignore_vars += (
                 "UACLIENT_BEHAVE_CONTRACT_TOKEN",
                 "UACLIENT_BEHAVE_CONTRACT_TOKEN_STAGING",
+                "UACLIENT_BEHAVE_CONTRACT_TOKEN_STAGING_EXPIRED",
             )
         for env_name in ignore_vars:
             attr_name = env_name.replace("UACLIENT_BEHAVE_", "").lower()
@@ -436,6 +463,35 @@ def before_scenario(context: Context, scenario: Scenario):
     reason = _should_skip_tags(context, scenario.effective_tags)
     if reason:
         scenario.skip(reason=reason)
+        return
+
+    filter_series = context.config.filter_series
+    given_a_series_match = re.match(
+        "a `(.*)` machine with ubuntu-advantage-tools installed",
+        scenario.steps[0].name,
+    )
+    if filter_series and given_a_series_match:
+        series = given_a_series_match.group(1)
+        if series and series not in filter_series:
+            scenario.skip(
+                reason=(
+                    "Skipping scenario outline series `{series}`."
+                    " Cmdline provided @series tags: {cmdline_series}".format(
+                        series=series, cmdline_series=filter_series
+                    )
+                )
+            )
+            return
+
+    if "uses.config.check_version" in scenario.effective_tags:
+        # before_step doesn't execute early enough to modify the step
+        # so we perform the version step surgery here
+        for step in scenario.steps:
+            if step.text:
+                step.text = step.text.replace(
+                    "{UACLIENT_BEHAVE_CHECK_VERSION}",
+                    context.config.check_version,
+                )
 
 
 FAILURE_FILES = (
@@ -500,27 +556,27 @@ def after_step(context, step):
                 )
                 print("-- pull instance:{} {}".format(log_file, artifact_file))
                 try:
-                    context.instance.pull_file(log_file, artifact_file)
-                except IOError as e:
-                    if e.errno == errno.EACCES:
-                        result = context.instance.execute(
-                            ["cat", log_file], use_sudo=True
-                        )
-                        with open(artifact_file, "w") as stream:
-                            stream.write(result.stdout)
+                    result = context.instances["uaclient"].execute(
+                        ["cat", log_file], use_sudo=True
+                    )
+                    content = result.stdout if result.ok else ""
                 except RuntimeError:
-                    # File did not exist
-                    with open(artifact_file, "w") as stream:
-                        stream.write("")
+                    content = ""
+                with open(artifact_file, "w") as stream:
+                    stream.write(content)
             for artifact_file, cmd in FAILURE_CMDS.items():
-                result = context.instance.execute(cmd, use_sudo=True)
+                result = context.instances["uaclient"].execute(
+                    cmd, use_sudo=True
+                )
                 artifact_file = os.path.join(artifacts_dir, artifact_file)
                 with open(artifact_file, "w") as stream:
                     stream.write(result.stdout)
 
 
 def after_all(context):
-    if context.config.image_clean:
+    if context.config.ppa == "":
+        print(" No custom images to delete. UACLIENT_BEHAVE_PPA is unset.")
+    elif context.config.image_clean:
         for key, image in context.series_image_name.items():
             if key == context.series_reuse_image:
                 print(
@@ -607,6 +663,7 @@ def build_debs_from_dev_instance(context: Context, series: str) -> "List[str]":
                 build_container_name,
                 output_deb_dir=os.path.join(tempfile.gettempdir(), series),
                 cloud_api=context.config.cloud_api,
+                cache_source=context.config.cache_source,
             )
 
     if "pro" in context.config.machine_type:
@@ -635,6 +692,16 @@ def create_uat_image(context: Context, series: str) -> None:
             context.reuse_container[series],
         )
         return
+    ppa = context.config.ppa
+    if ppa == "":
+        image_name = context.config.cloud_manager.locate_image_name(series)
+        print(
+            "--- Unset UACLIENT_BEHAVE_PPA. Using ubuntu-advantage-tools "
+            + "from image: {}".format(image_name)
+        )
+        context.series_image_name[series] = image_name
+        return
+
     time_suffix = datetime.datetime.now().strftime("%s%f")
     deb_paths = []
     if context.config.build_pr:
@@ -651,32 +718,33 @@ def create_uat_image(context: Context, series: str) -> None:
     )
 
     user_data = ""
+    ud_file = context.config.userdata_file
+    if ud_file and os.path.exists(ud_file):
+        with open(ud_file, "r") as stream:
+            user_data = stream.read()
     if "pro" in context.config.machine_type:
-        user_data = USERDATA_BLOCK_AUTO_ATTACH_IMG
+        if not user_data:
+            user_data = USERDATA_BLOCK_AUTO_ATTACH_IMG
     if not deb_paths:
         if not user_data:
             user_data = "#cloud-config\n"
-        ppa = context.config.ppa
-        ppa_keyid = context.config.ppa_keyid
-        if context.config.ppa.startswith("ppa:"):
-            ppa = ppa.replace("ppa:", "http://ppa.launchpad.net/") + "/ubuntu"
-        if series == "trusty":
-            packages = ["ubuntu-advantage-tools"]
 
-            if "pro" in context.config.machine_type:
-                packages.append("ubuntu-advantage-pro")
-
-            user_data += USERDATA_APT_SOURCE_PPA_TRUSTY.format(
-                ppa_url=ppa, ppa_keyid=ppa_keyid, packages=", ".join(packages)
-            )
+        if context.config.enable_proposed:
+            user_data += USERDATA_RUNCMD_ENABLE_PROPOSED.format(series=series)
         else:
-            packages = ["openssh-server", "ubuntu-advantage-tools"]
+            ppa = context.config.ppa
+            ppa_keyid = context.config.ppa_keyid
+            if context.config.ppa.startswith("ppa:"):
+                ppa = (
+                    ppa.replace("ppa:", "http://ppa.launchpad.net/")
+                    + "/ubuntu"
+                )
 
-            if "pro" in context.config.machine_type:
-                packages.append("ubuntu-advantage-pro")
+            if ppa == DAILY_PPA:
+                user_data += USERDATA_RUNCMD_PIN_PPA
 
             user_data += USERDATA_APT_SOURCE_PPA.format(
-                ppa_url=ppa, ppa_keyid=ppa_keyid, packages=", ".join(packages)
+                ppa_url=ppa, ppa_keyid=ppa_keyid
             )
     inst = context.config.cloud_manager.launch(
         instance_name=build_container_name, series=series, user_data=user_data
@@ -687,6 +755,7 @@ def create_uat_image(context: Context, series: str) -> None:
         build_container_name,
         series=series,
         config=context.config,
+        machine_type=context.config.machine_type,
         deb_paths=deb_paths,
     )
 
@@ -703,6 +772,7 @@ def _install_uat_in_container(
     container_name: str,
     series: str,
     config: UAClientBehaveConfig,
+    machine_type: str,
     deb_paths: "Optional[List[str]]" = None,
 ) -> None:
     """Install ubuntu-advantage-tools into the specified container
@@ -733,19 +803,53 @@ def _install_uat_in_container(
             deb_files.append("/tmp/" + deb_name)
             inst.push_file(deb_file, "/tmp/" + deb_name)
 
-        if series == "trusty":
-            cmds.append(["sudo", "dpkg", "-i"] + deb_files)
-        else:
-            cmds.append(["sudo", "apt-get", "install", "-y"] + deb_files)
+        cmds.append(
+            " ".join(
+                [
+                    "sudo",
+                    "DEBIAN_FRONTEND=noninteractive",
+                    "apt-get",
+                    "install",
+                    "-y",
+                    "--allow-downgrades",
+                    '-o Dpkg::Options::="--force-confdef"',
+                    '-o Dpkg::Options::="--force-confold"',
+                ]
+                + deb_files
+            )
+        )
+    else:
+        ua_pkg = ["ubuntu-advantage-tools"]
+        if "pro" in machine_type:
+            ua_pkg.append("ubuntu-advantage-pro")
+
+        cmds.append(
+            " ".join(
+                [
+                    "sudo",
+                    "DEBIAN_FRONTEND=noninteractive",
+                    "apt-get",
+                    "install",
+                    "-y",
+                    "--allow-downgrades",
+                    '-o Dpkg::Options::="--force-confdef"',
+                    '-o Dpkg::Options::="--force-confold"',
+                ]
+                + ua_pkg
+            )
+        )
 
     if "pro" in config.machine_type:
         features = "features:\n  disable_auto_attach: true\n"
         conf_path = "/etc/ubuntu-advantage/uaclient.conf"
-        cmd = "printf '{}' > /tmp/uaclient.conf".format(features)
+        cmd = "printf '{}' > /var/tmp/uaclient.conf".format(features)
         cmds.append('sh -c "{}"'.format(cmd))
         cmds.append(
-            'sudo -- sh -c "cat /tmp/uaclient.conf >> {}"'.format(conf_path)
+            'sudo -- sh -c "cat /var/tmp/uaclient.conf >> {}"'.format(
+                conf_path
+            )
         )
+        cmds.append("sudo ua status --wait")
         cmds.append("sudo ua detach --assume-yes")
 
     cmds.append(["ua", "version"])
