@@ -18,7 +18,7 @@ from hamcrest import (
 from features.environment import create_uat_image
 from features.util import SLOW_CMDS, emit_spinner_on_travis, nullcontext
 
-from uaclient.defaults import DEFAULT_CONFIG_FILE
+from uaclient.defaults import DEFAULT_CONFIG_FILE, DEFAULT_MACHINE_TOKEN_PATH
 
 
 CONTAINER_PREFIX = "ubuntu-behave-test-"
@@ -26,26 +26,16 @@ CONTAINER_PREFIX = "ubuntu-behave-test-"
 
 @given("a `{series}` machine with ubuntu-advantage-tools installed")
 def given_a_machine(context, series):
-    filter_series = context.config.filter_series
-    if filter_series and series not in filter_series:
-        context.scenario.skip(
-            reason=(
-                "Skipping scenario outline series {series}."
-                " Cmdline provided @series tags: {cmdline_series}".format(
-                    series=series, cmdline_series=filter_series
-                )
-            )
-        )
-        return
     if series in context.reuse_container:
+        context.instances = {}
         context.container_name = context.reuse_container[series]
-        context.instance = context.config.cloud_api.get_instance(
+        context.instances["uaclient"] = context.config.cloud_api.get_instance(
             context.container_name
         )
         if "pro" in context.config.machine_type:
-            context.instance = context.config.cloud_api.get_instance(
-                context.container_name
-            )
+            context.instances[
+                "uaclient"
+            ] = context.config.cloud_api.get_instance(context.container_name)
         return
 
     if series not in context.series_image_name:
@@ -64,36 +54,92 @@ def given_a_machine(context, series):
         CONTAINER_PREFIX + pr_prefix + vm_prefix + series + date_prefix
     )
 
-    context.instance = context.config.cloud_manager.launch(
-        series=series,
-        instance_name=instance_name,
-        image_name=context.series_image_name[series],
-    )
+    context.instances = {
+        "uaclient": context.config.cloud_manager.launch(
+            series=series,
+            instance_name=instance_name,
+            image_name=context.series_image_name[series],
+        )
+    }
 
     context.container_name = context.config.cloud_manager.get_instance_id(
-        context.instance
+        context.instances["uaclient"]
     )
 
     def cleanup_instance() -> None:
         if not context.config.destroy_instances:
             print(
                 "--- Leaving instance running: {}".format(
-                    context.instance.name
+                    context.instances["uaclient"].name
                 )
             )
         else:
             try:
-                context.instance.delete(wait=False)
+                context.instances["uaclient"].delete(wait=False)
             except RuntimeError as e:
                 print(
                     "Failed to delete instance: {}\n{}".format(
-                        context.instance.name, str(e)
+                        context.instances["uaclient"].name, str(e)
                     )
                 )
 
     context.add_cleanup(cleanup_instance)
-    logging.warning("--- instance ip: {}".format(context.instance.ip))
-    print("--- instance ip: {}".format(context.instance.ip))
+    logging.warning(
+        "--- instance ip: {}".format(context.instances["uaclient"].ip)
+    )
+    print("--- instance ip: {}".format(context.instances["uaclient"].ip))
+
+
+@when("I launch a `{series}` `{instance_name}` machine")
+def launch_machine(context, series, instance_name):
+    now = datetime.datetime.now()
+    date_prefix = now.strftime("-%s%f")
+    name = CONTAINER_PREFIX + series + date_prefix + "-" + instance_name
+
+    context.instances[instance_name] = context.config.cloud_manager.launch(
+        series=series, instance_name=name
+    )
+
+    def cleanup_instance() -> None:
+        try:
+            context.instances[instance_name].delete(wait=False)
+        except RuntimeError as e:
+            print(
+                "Failed to delete instance: {}\n{}".format(
+                    context.instances[instance_name].name, str(e)
+                )
+            )
+
+    context.add_cleanup(cleanup_instance)
+
+
+@when("I add this text on `{file_name}` on `{instance_name}` above `{line}`")
+def when_i_add_this_text_on_file_above_line(
+    context, file_name, instance_name, line
+):
+    command = 'sed -i "s/{}/{}\\n{}/" {}'.format(
+        line, context.text, line, file_name
+    )
+    when_i_run_command(
+        context, command, "with sudo", instance_name=instance_name
+    )
+
+
+@when("I run `{command}` `{user_spec}` on the `{instance_name}` machine")
+def when_i_run_command_on_machine(context, command, user_spec, instance_name):
+    when_i_run_command(
+        context, command, user_spec, instance_name=instance_name
+    )
+
+
+@when("I verify `{file_name}` is empty on `{instance_name}` machine")
+def when_i_verify_file_is_empty_on_machine(context, file_name, instance_name):
+    command = 'sh -c "cat {} | wc -l"'
+    when_i_run_command(
+        context, command, user_spec="with sudo", instance_name=instance_name
+    )
+
+    assert_that(context.process.stdout.strip(), matches_regexp("0"))
 
 
 @when("I run `{command}` {user_spec}, retrying exit [{exit_codes}]")
@@ -118,7 +164,18 @@ def when_i_retry_run_command(context, command, user_spec, exit_codes):
 
 
 @when("I run `{command}` {user_spec}")
-def when_i_run_command(context, command, user_spec, verify_return=True):
+def when_i_run_command(
+    context,
+    command,
+    user_spec,
+    verify_return=True,
+    stdin=None,
+    instance_name="uaclient",
+):
+    if "<ci-proxy-ip>" in command:
+        command = command.replace(
+            "<ci-proxy-ip>", context.instances["proxy"].ip
+        )
     prefix = get_command_prefix_for_user_spec(user_spec)
     slow_cmd_spinner = nullcontext
     for slow_cmd in SLOW_CMDS:
@@ -128,7 +185,9 @@ def when_i_run_command(context, command, user_spec, verify_return=True):
 
     full_cmd = prefix + shlex.split(command)
     with slow_cmd_spinner():
-        result = context.instance.execute(full_cmd)
+        result = context.instances[instance_name].execute(
+            full_cmd, stdin=stdin
+        )
 
     process = subprocess.CompletedProcess(
         args=full_cmd,
@@ -148,10 +207,68 @@ def when_i_run_command(context, command, user_spec, verify_return=True):
     context.process = process
 
 
-@when("I attach `{token_type}` {user_spec}")
-def when_i_attach_staging_token(context, token_type, user_spec):
+@when("I fix `{issue}` by attaching to a subscription with `{token_type}`")
+def when_i_fix_a_issue_by_attaching(context, issue, token_type):
     token = getattr(context.config, token_type)
-    if token_type == "contract_token_staging":
+    when_i_run_command(
+        context=context,
+        command="ua fix {}".format(issue),
+        user_spec="with sudo",
+        stdin="a\n{}\n".format(token),
+    )
+
+
+@when("I fix `{issue}` by enabling required service")
+def when_i_fix_a_issue_by_enabling_service(context, issue):
+    when_i_run_command(
+        context=context,
+        command="ua fix {}".format(issue),
+        user_spec="with sudo",
+        stdin="e\n",
+    )
+
+
+@when("I fix `{issue}` by updating expired token")
+def when_i_fix_a_issue_by_updating_expired_token(context, issue):
+    token = getattr(context.config, "contract_token")
+    when_i_run_command(
+        context=context,
+        command="ua fix {}".format(issue),
+        user_spec="with sudo",
+        stdin="r\n{}\n".format(token),
+    )
+
+
+@when("I update contract to use `{contract_field}` as `{new_value}`")
+def when_i_update_contract_field_to_new_value(
+    context, contract_field, new_value
+):
+    if contract_field == "effectiveTo":
+        if "days=" in new_value:  # Set timedelta offset from current day
+            now = datetime.datetime.utcnow()
+            contract_expiry = now + datetime.timedelta(days=int(new_value[5:]))
+            new_value = contract_expiry.strftime("%Y-%m-%dT00:00:00Z")
+    when_i_run_command(
+        context,
+        'sed -i \'s/"{}": "[^"]*"/"{}": "{}"/g\' {}'.format(
+            contract_field,
+            contract_field,
+            new_value,
+            DEFAULT_MACHINE_TOKEN_PATH,
+        ),
+        user_spec="with sudo",
+    )
+
+
+@when("I attach `{token_type}` {user_spec}")
+def when_i_attach_staging_token(
+    context, token_type, user_spec, verify_return=True
+):
+    token = getattr(context.config, token_type)
+    if (
+        token_type == "contract_token_staging"
+        or token_type == "contract_token_staging_expired"
+    ):
         when_i_run_command(
             context,
             "sed -i 's/contracts.can/contracts.staging.can/' {}".format(
@@ -162,19 +279,27 @@ def when_i_attach_staging_token(context, token_type, user_spec):
     cmd = "ua attach {}".format(token)
     when_i_run_command(context, cmd, user_spec, verify_return=False)
 
-    retries = [5, 5, 10]  # Sleep times to wait between retries
-    while context.process.returncode != 0:
-        try:
-            time.sleep(retries.pop(0))
-        except IndexError:  # no more timeouts
-            logging.warning("Exhausted retries waiting for exit code: 0")
-            break
-        print(
-            "--- Retrying on exit {exit_code}: {cmd}".format(
-                exit_code=context.process.returncode, cmd=cmd
+    if verify_return:
+        retries = [5, 5, 10]  # Sleep times to wait between retries
+        while context.process.returncode != 0:
+            try:
+                time.sleep(retries.pop(0))
+            except IndexError:  # no more timeouts
+                logging.warning("Exhausted retries waiting for exit code: 0")
+                break
+            print(
+                "--- Retrying on exit {exit_code}: {cmd}".format(
+                    exit_code=context.process.returncode, cmd=cmd
+                )
             )
-        )
-        when_i_run_command(context, cmd, user_spec, verify_return=False)
+            when_i_run_command(context, cmd, user_spec, verify_return=False)
+
+
+@when("I attempt to attach `{token_type}` {user_spec}")
+def when_i_attempt_to_attach_staging_token(context, token_type, user_spec):
+    when_i_attach_staging_token(
+        context, token_type, user_spec, verify_return=False
+    )
 
 
 @when("I append the following on uaclient config")
@@ -191,7 +316,8 @@ def when_i_append_to_uaclient_config(context):
 @when("I create the file `{file_path}` with the following")
 def when_i_create_file_with_content(context, file_path):
     text = context.text.replace('"', '\\"')
-
+    if "<ci-proxy-ip>" in text:
+        text = text.replace("<ci-proxy-ip>", context.instances["proxy"].ip)
     cmd = "printf '{}\n' > {}".format(text, file_path)
     cmd = 'sh -c "{}"'.format(cmd)
     when_i_run_command(context, cmd, "with sudo")
@@ -199,18 +325,7 @@ def when_i_create_file_with_content(context, file_path):
 
 @when("I reboot the `{series}` machine")
 def when_i_reboot_the_machine(context, series):
-    if series == "trusty":
-        # TODO(LP: #1899299: LTS upgrade T->X pickled ds breaks Paths.run_dir)
-        # When Fix is SRUd to Xenial, we can drop the trusty clause
-        logging.warning(
-            "LP: #1899299: Not raising cloud-init-errors across Trusty reboot"
-        )
-        context.instance.shutdown(wait=True)
-        context.instance.start(wait=False)
-        # Trusty -> Xenial upgrades would raise a Paths no run_dir attr failure
-        context.instance.wait()
-    else:
-        context.instance.restart(wait=True)
+    context.instances["uaclient"].restart(wait=True)
 
 
 @then("I will see the following on stdout")
@@ -225,6 +340,13 @@ def then_conditional_stdout_matches_regexp(context, value1, value2):
         then_stdout_matches_regexp(context)
 
 
+@then("if `{value1}` in `{value2}` and stdout does not match regexp")
+def then_conditional_stdout_does_not_match_regexp(context, value1, value2):
+    """Only apply regex assertion if value1 in value2."""
+    if value1 in value2.split(" or "):
+        then_stdout_does_not_match_regexp(context)
+
+
 @then("stdout matches regexp")
 def then_stdout_matches_regexp(context):
     assert_that(context.process.stdout.strip(), matches_regexp(context.text))
