@@ -97,24 +97,36 @@ def entitlement_factory(tmpdir):
         directives: "Dict[str, Any]" = None,
         obligations: "Dict[str, Any]" = None,
         entitled: bool = True,
+        allow_beta: bool = False,
         assume_yes: "Optional[bool]" = None,
         suites: "List[str]" = None,
-        additional_packages: "List[str]" = None
+        additional_packages: "List[str]" = None,
+        services_once_enabled: "Dict[str, bool]" = None,
+        cfg: "Optional[config.UAConfig]" = None,
+        cfg_extension: "Optional[Dict[str, Any]]" = None
     ):
-        cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
-        cfg.write_cache(
-            "machine-token",
-            machine_token(
-                cls.name,
-                affordances=affordances,
-                directives=directives,
-                obligations=obligations,
-                entitled=entitled,
-                suites=suites,
-                additional_packages=additional_packages,
-            ),
-        )
-        args = {}
+        if not cfg:
+            cfg_arg = {"data_dir": tmpdir.strpath}
+            if cfg_extension is not None:
+                cfg_arg.update(cfg_extension)
+            cfg = config.UAConfig(cfg=cfg_arg)
+            cfg.write_cache(
+                "machine-token",
+                machine_token(
+                    cls.name,
+                    affordances=affordances,
+                    directives=directives,
+                    obligations=obligations,
+                    entitled=entitled,
+                    suites=suites,
+                    additional_packages=additional_packages,
+                ),
+            )
+
+        if services_once_enabled:
+            cfg.write_cache("services-once-enabled", services_once_enabled)
+
+        args = {"allow_beta": allow_beta}
         if assume_yes is not None:
             args["assume_yes"] = assume_yes
         return cls(cfg, **args)
