@@ -11,7 +11,7 @@ from uaclient import util
 from uaclient.status import ContractStatus
 
 try:
-    from typing import Tuple  # noqa
+    from typing import Dict, Optional, Tuple  # noqa
 except ImportError:
     pass
 
@@ -29,8 +29,9 @@ class ConcreteTestEntitlement(base.UAEntitlement):
         enable=None,
         applicability_status=None,
         application_status=None,
+        allow_beta=False,
     ):
-        super().__init__(cfg)
+        super().__init__(cfg, allow_beta=allow_beta)
         self._disable = disable
         self._enable = enable
         self._applicability_status = applicability_status
@@ -43,7 +44,7 @@ class ConcreteTestEntitlement(base.UAEntitlement):
         )
         return self._disable
 
-    def enable(self, silent_if_inapplicable: bool = False):
+    def _perform_enable(self):
         return self._enable
 
     def applicability_status(self):
@@ -59,7 +60,10 @@ def concrete_entitlement_factory(tmpdir):
         *,
         entitled: bool,
         applicability_status: "Tuple[status.ApplicabilityStatus, str]" = None,
-        application_status: "Tuple[status.ApplicationStatus, str]" = None
+        application_status: "Tuple[status.ApplicationStatus, str]" = None,
+        feature_overrides: "Optional[Dict[str, str]]" = None,
+        allow_beta: bool = False,
+        enable: bool = False
     ) -> ConcreteTestEntitlement:
         cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
         machineToken = {
@@ -76,10 +80,16 @@ def concrete_entitlement_factory(tmpdir):
             },
         }
         cfg.write_cache("machine-token", machineToken)
+
+        if feature_overrides:
+            cfg.cfg.update({"features": feature_overrides})
+
         return ConcreteTestEntitlement(
             cfg,
             applicability_status=applicability_status,
             application_status=application_status,
+            allow_beta=allow_beta,
+            enable=enable,
         )
 
     return factory
@@ -92,8 +102,8 @@ class TestUaEntitlement:
             base.UAEntitlement()
         expected_msg = (
             "Can't instantiate abstract class UAEntitlement with abstract"
-            " methods application_status, description, disable, enable, name,"
-            " title"
+            " methods _perform_enable, application_status, description,"
+            " disable, name, title"
         )
         assert expected_msg == str(excinfo.value)
 
@@ -140,27 +150,18 @@ class TestUaEntitlement:
         stdout, _ = capsys.readouterr()
         assert "" == stdout
 
-    @pytest.mark.parametrize("silent", (True, False, None))
     def test_can_enable_false_on_unentitled(
-        self, capsys, concrete_entitlement_factory, silent
+        self, concrete_entitlement_factory
     ):
         """When entitlement contract is not enabled, can_enable is False."""
-
         entitlement = concrete_entitlement_factory(entitled=False)
 
-        kwargs = {}
-        if silent is not None:
-            kwargs["silent"] = silent
-        assert not entitlement.can_enable(**kwargs)
-
-        expected_stdout = (
-            "This subscription is not entitled to Test Concrete Entitlement.\n"
-            "For more information see: https://ubuntu.com/advantage\n"
+        can_enable, reason = entitlement.can_enable()
+        assert not can_enable
+        assert reason.reason == status.CanEnableFailureReason.NOT_ENTITLED
+        assert reason.message == status.MESSAGE_UNENTITLED_TMPL.format(
+            title=ConcreteTestEntitlement.title
         )
-        if silent:
-            expected_stdout = ""
-        stdout, _ = capsys.readouterr()
-        assert expected_stdout == stdout
 
     @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
     @mock.patch("uaclient.contract.request_updated_contract")
@@ -168,7 +169,6 @@ class TestUaEntitlement:
         self,
         m_request_updated_contract,
         caplog_text,
-        capsys,
         concrete_entitlement_factory,
     ):
         """When entitlement contract is not enabled, can_enable is False."""
@@ -176,7 +176,7 @@ class TestUaEntitlement:
         ent = concrete_entitlement_factory(entitled=False)
 
         with mock.patch.object(ent, "is_access_expired", return_value=True):
-            assert not ent.can_enable()
+            assert not ent.can_enable()[0]
 
         assert [
             mock.call(ent.cfg)
@@ -186,9 +186,8 @@ class TestUaEntitlement:
             in caplog_text()
         )
 
-    @pytest.mark.parametrize("silent", (True, False, None))
     def test_can_enable_false_on_entitlement_active(
-        self, capsys, concrete_entitlement_factory, silent
+        self, concrete_entitlement_factory
     ):
         """When entitlement is ENABLED, can_enable returns False."""
         application_status = status.ApplicationStatus.ENABLED
@@ -196,23 +195,15 @@ class TestUaEntitlement:
             entitled=True, application_status=(application_status, "")
         )
 
-        kwargs = {}
-        if silent is not None:
-            kwargs["silent"] = silent
-        assert not entitlement.can_enable(**kwargs)
-
-        expected_stdout = (
-            "Test Concrete Entitlement is already enabled.\n"
-            "See: sudo ua status\n"
+        can_enable, reason = entitlement.can_enable()
+        assert not can_enable
+        assert reason.reason == status.CanEnableFailureReason.ALREADY_ENABLED
+        assert reason.message == status.MESSAGE_ALREADY_ENABLED_TMPL.format(
+            title=ConcreteTestEntitlement.title
         )
-        if silent:
-            expected_stdout = ""
-        stdout, _ = capsys.readouterr()
-        assert expected_stdout == stdout
 
-    @pytest.mark.parametrize("silent", (True, False, None))
     def test_can_enable_false_on_entitlement_inapplicable(
-        self, capsys, concrete_entitlement_factory, silent
+        self, concrete_entitlement_factory
     ):
         """When entitlement is INAPPLICABLE, can_enable returns False."""
         entitlement = concrete_entitlement_factory(
@@ -224,20 +215,13 @@ class TestUaEntitlement:
             application_status=(status.ApplicationStatus.DISABLED, ""),
         )
 
-        kwargs = {}
-        if silent is not None:
-            kwargs["silent"] = silent
-        assert not entitlement.can_enable(**kwargs)
+        can_enable, reason = entitlement.can_enable()
+        assert not can_enable
+        assert reason.reason == status.CanEnableFailureReason.INAPPLICABLE
+        assert reason.message == "msg"
 
-        expected_stdout = "msg\n"
-        if silent:
-            expected_stdout = ""
-        stdout, _ = capsys.readouterr()
-        assert expected_stdout == stdout
-
-    @pytest.mark.parametrize("silent", (True, False, None))
     def test_can_enable_true_on_entitlement_inactive(
-        self, capsys, concrete_entitlement_factory, silent
+        self, concrete_entitlement_factory
     ):
         """When an entitlement is applicable and disabled, we can_enable"""
         entitlement = concrete_entitlement_factory(
@@ -246,13 +230,35 @@ class TestUaEntitlement:
             application_status=(status.ApplicationStatus.DISABLED, ""),
         )
 
-        kwargs = {}
-        if silent is not None:
-            kwargs["silent"] = silent
-        assert entitlement.can_enable(**kwargs)
+        can_enable, reason = entitlement.can_enable()
+        assert can_enable
+        assert reason is None
 
-        stdout, _ = capsys.readouterr()
-        assert "" == stdout
+    @pytest.mark.parametrize("is_beta", (True, False))
+    @pytest.mark.parametrize("allow_beta", (True, False))
+    @pytest.mark.parametrize("allow_beta_cfg", (True, False))
+    def test_can_enable_on_beta_service(
+        self, allow_beta_cfg, allow_beta, is_beta, concrete_entitlement_factory
+    ):
+
+        feature_overrides = {"allow_beta": allow_beta_cfg}
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(status.ApplicationStatus.DISABLED, ""),
+            feature_overrides=feature_overrides,
+            allow_beta=allow_beta,
+        )
+        entitlement.is_beta = is_beta
+        can_enable, reason = entitlement.can_enable()
+
+        if not is_beta or allow_beta or allow_beta_cfg:
+            assert can_enable
+            assert reason is None
+        else:
+            assert not can_enable
+            assert reason.reason == status.CanEnableFailureReason.IS_BETA
+            assert reason.message is None
 
     def test_contract_status_entitled(self, concrete_entitlement_factory):
         """The contract_status returns ENTITLED when entitlement enabled."""
@@ -281,24 +287,11 @@ class TestUaEntitlement:
             entitlement.process_contract_deltas(orig_access, delta)
         assert 0 == m_can_disable.call_count
 
-    @pytest.mark.parametrize(
-        "block_disable_on_enable,assume_yes",
-        ((True, True), (False, False), (True, False), (False, True)),
-    )
-    @mock.patch("uaclient.util.is_config_value_true")
-    @mock.patch("uaclient.util.prompt_for_confirmation")
     def test_can_enable_when_incompatible_service_found(
-        self,
-        m_prompt,
-        m_is_config_value_true,
-        block_disable_on_enable,
-        assume_yes,
-        concrete_entitlement_factory,
+        self, concrete_entitlement_factory
     ):
         import uaclient.entitlements as ent
 
-        m_prompt.return_value = assume_yes
-        m_is_config_value_true.return_value = block_disable_on_enable
         base_ent = concrete_entitlement_factory(
             entitled=True,
             applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
@@ -320,19 +313,134 @@ class TestUaEntitlement:
             with mock.patch.object(
                 ent, "ENTITLEMENT_CLASS_BY_NAME", {"test": m_entitlement_cls}
             ):
-                ret = base_ent.can_enable()
+                ret, reason = base_ent.can_enable()
+
+        assert ret is False
+        assert (
+            reason.reason == status.CanEnableFailureReason.INCOMPATIBLE_SERVICE
+        )
+        assert reason.message is None
+
+    @pytest.mark.parametrize(
+        "block_disable_on_enable,assume_yes",
+        [(False, False), (False, True), (True, False), (True, True)],
+    )
+    @mock.patch("uaclient.util.is_config_value_true")
+    @mock.patch("uaclient.util.prompt_for_confirmation")
+    def test_enable_when_incompatible_service_found(
+        self,
+        m_prompt,
+        m_is_config_value_true,
+        block_disable_on_enable,
+        assume_yes,
+        concrete_entitlement_factory,
+    ):
+        import uaclient.entitlements as ent
+
+        m_prompt.return_value = assume_yes
+        m_is_config_value_true.return_value = block_disable_on_enable
+        base_ent = concrete_entitlement_factory(
+            entitled=True,
+            enable=True,
+            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(status.ApplicationStatus.DISABLED, ""),
+        )
+        base_ent._incompatible_services = ["test"]
+
+        m_entitlement_cls = mock.MagicMock()
+        m_entitlement_obj = m_entitlement_cls.return_value
+        m_entitlement_obj.application_status.return_value = [
+            status.ApplicationStatus.ENABLED,
+            "",
+        ]
+        type(m_entitlement_obj).title = mock.PropertyMock(return_value="test")
+
+        with mock.patch.object(
+            base_ent, "is_access_expired", return_value=False
+        ):
+            with mock.patch.object(
+                ent, "ENTITLEMENT_CLASS_BY_NAME", {"test": m_entitlement_cls}
+            ):
+                ret, reason = base_ent.enable()
+
+        expected_prompt_call = 1
+        if block_disable_on_enable:
+            expected_prompt_call = 0
 
-        expected_prompt_call = 0 if block_disable_on_enable else 1
         expected_ret = False
+        expected_reason = status.CanEnableFailureReason.INCOMPATIBLE_SERVICE
         if assume_yes and not block_disable_on_enable:
             expected_ret = True
+            expected_reason = None
         expected_disable_call = 1 if expected_ret else 0
 
         assert ret == expected_ret
+        if expected_reason is None:
+            assert reason is None
+        else:
+            assert reason.reason == expected_reason
         assert m_prompt.call_count == expected_prompt_call
         assert m_is_config_value_true.call_count == 1
         assert m_entitlement_obj.disable.call_count == expected_disable_call
 
+    @pytest.mark.parametrize(
+        "can_enable_fail,handle_incompat_calls",
+        [
+            (
+                status.CanEnableFailure(
+                    status.CanEnableFailureReason.NOT_ENTITLED, message="msg"
+                ),
+                0,
+            ),
+            (
+                status.CanEnableFailure(
+                    status.CanEnableFailureReason.ALREADY_ENABLED,
+                    message="msg",
+                ),
+                0,
+            ),
+            (
+                status.CanEnableFailure(status.CanEnableFailureReason.IS_BETA),
+                0,
+            ),
+            (
+                status.CanEnableFailure(
+                    status.CanEnableFailureReason.INAPPLICABLE, "msg"
+                ),
+                0,
+            ),
+            (
+                status.CanEnableFailure(
+                    status.CanEnableFailureReason.INCOMPATIBLE_SERVICE
+                ),
+                1,
+            ),
+        ],
+    )
+    @mock.patch(
+        "uaclient.entitlements.base.UAEntitlement.handle_incompatible_services",  # noqa: E501
+        return_value=False,
+    )
+    @mock.patch("uaclient.entitlements.base.UAEntitlement.can_enable")
+    def test_enable_false_when_can_enable_false(
+        self,
+        m_can_enable,
+        m_handle_incompat,
+        can_enable_fail,
+        handle_incompat_calls,
+        concrete_entitlement_factory,
+    ):
+        """When can_enable returns False enable returns False."""
+        m_can_enable.return_value = (False, can_enable_fail)
+        entitlement = concrete_entitlement_factory(entitled=True)
+        entitlement._perform_enable = mock.Mock()
+
+        assert (False, can_enable_fail) == entitlement.enable()
+
+        assert 1 == m_can_enable.call_count
+        assert handle_incompat_calls == m_handle_incompat.call_count
+        assert 0 == entitlement._perform_enable.call_count
+
     @pytest.mark.parametrize(
         "orig_access,delta",
         (
@@ -431,6 +539,45 @@ class TestUaEntitlement:
             mock.ANY,
         ) == entitlement.application_status()
 
+    @pytest.mark.parametrize(
+        "orig_access,delta",
+        (
+            (
+                {
+                    "resourceToken": "test",
+                    "entitlement": {
+                        "entitled": True,
+                        "obligations": {"enableByDefault": False},
+                    },
+                },
+                {
+                    "entitlement": {
+                        "entitled": True,
+                        "obligations": {"enableByDefault": True},
+                    }
+                },
+            ),
+        ),
+    )
+    def test_process_contract_deltas_enable_beta_if_enabled_by_default_turned(
+        self, concrete_entitlement_factory, orig_access, delta
+    ):
+        """Disable when deltas transition from active to unentitled."""
+        entitlement = concrete_entitlement_factory(
+            entitled=True,
+            applicability_status=(status.ApplicabilityStatus.APPLICABLE, ""),
+            application_status=(status.ApplicationStatus.DISABLED, ""),
+        )
+        entitlement.is_beta = True
+        assert not entitlement.allow_beta
+        with mock.patch.object(entitlement, "enable") as m_enable:
+            entitlement.process_contract_deltas(
+                orig_access, delta, allow_enable=True
+            )
+            assert 1 == m_enable.call_count
+
+        assert entitlement.allow_beta
+
 
 class TestUaEntitlementUserFacingStatus:
     def test_inapplicable_when_not_applicable(
