@@ -52,7 +52,7 @@ class TestCommonCriteriaEntitlementUserFacingStatus:
                 "xenial",
                 "16.04 LTS (Xenial Xerus)",
                 "CC EAL2 is not available for platform arm64.\n"
-                "Supported platforms are: x86_64, ppc64le, s390x",
+                "Supported platforms are: x86_64, ppc64le, s390x.",
             ),
             (
                 "s390x",
@@ -92,12 +92,12 @@ class TestCommonCriteriaEntitlementCanEnable:
         m_platform_info.return_value = PLATFORM_INFO_SUPPORTED
         cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
         cfg.write_cache("machine-token", CC_MACHINE_TOKEN)
-        entitlement = CommonCriteriaEntitlement(cfg)
+        entitlement = CommonCriteriaEntitlement(cfg, allow_beta=True)
         uf_status, uf_status_details = entitlement.user_facing_status()
         assert status.UserFacingStatus.INACTIVE == uf_status
         details = "{} is not configured".format(entitlement.title)
         assert details == uf_status_details
-        assert True is entitlement.can_enable()
+        assert (True, None) == entitlement.can_enable()
         assert ("", "") == capsys.readouterr()
 
 
@@ -108,6 +108,7 @@ class TestCommonCriteriaEntitlementEnable:
         "apt_transport_https,ca_certificates",
         itertools.product([False, True], repeat=2),
     )
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch("uaclient.util.should_reboot")
     @mock.patch("uaclient.util.subp")
     @mock.patch("uaclient.util.get_platform_info")
@@ -116,6 +117,7 @@ class TestCommonCriteriaEntitlementEnable:
         m_platform_info,
         m_subp,
         m_should_reboot,
+        m_setup_apt_proxy,
         capsys,
         tmpdir,
         apt_transport_https,
@@ -145,14 +147,14 @@ class TestCommonCriteriaEntitlementEnable:
         m_platform_info.side_effect = fake_platform
         cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
         cfg.write_cache("machine-token", CC_MACHINE_TOKEN)
-        entitlement = CommonCriteriaEntitlement(cfg)
+        entitlement = CommonCriteriaEntitlement(cfg, allow_beta=True)
 
         with mock.patch("uaclient.apt.add_auth_apt_repo") as m_add_apt:
             with mock.patch("uaclient.apt.add_ppa_pinning") as m_add_pin:
                 with mock.patch(
                     M_REPOPATH + "os.path.exists", side_effect=exists
                 ):
-                    assert True is entitlement.enable()
+                    assert (True, None) == entitlement.enable()
 
         add_apt_calls = [
             mock.call(
@@ -215,6 +217,7 @@ class TestCommonCriteriaEntitlementEnable:
         assert add_apt_calls == m_add_apt.call_args_list
         # No apt pinning for cc
         assert [] == m_add_pin.call_args_list
+        assert 1 == m_setup_apt_proxy.call_count
         assert 1 == m_should_reboot.call_count
         assert subp_apt_cmds == m_subp.call_args_list
         expected_stdout += "\n".join(
