@@ -6,7 +6,7 @@ import pytest
 
 from uaclient import apt
 from uaclient import status
-from uaclient.entitlements.cis import CISEntitlement
+from uaclient.entitlements.cis import CISEntitlement, CIS_DOCS_URL
 
 
 M_REPOPATH = "uaclient.entitlements.repo."
@@ -14,7 +14,9 @@ M_REPOPATH = "uaclient.entitlements.repo."
 
 @pytest.fixture
 def entitlement(entitlement_factory):
-    return entitlement_factory(CISEntitlement, additional_packages=["pkg1"])
+    return entitlement_factory(
+        CISEntitlement, allow_beta=True, additional_packages=["pkg1"]
+    )
 
 
 class TestCISEntitlementCanEnable:
@@ -33,11 +35,18 @@ class TestCISEntitlementCanEnable:
 
 
 class TestCISEntitlementEnable:
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch("uaclient.util.should_reboot")
     @mock.patch("uaclient.util.subp")
     @mock.patch("uaclient.util.get_platform_info")
     def test_enable_configures_apt_sources_and_auth_files(
-        self, m_platform_info, m_subp, m_should_reboot, capsys, entitlement
+        self,
+        m_platform_info,
+        m_subp,
+        m_should_reboot,
+        m_setup_apt_proxy,
+        capsys,
+        entitlement,
     ):
         """When entitled, configure apt repo auth token, pinning and url."""
 
@@ -86,6 +95,7 @@ class TestCISEntitlementEnable:
                     "apt-get",
                     "install",
                     "--assume-yes",
+                    "--allow-downgrades",
                     '-o Dpkg::Options::="--force-confdef"',
                     '-o Dpkg::Options::="--force-confold"',
                 ]
@@ -99,11 +109,13 @@ class TestCISEntitlementEnable:
         assert add_apt_calls == m_add_apt.call_args_list
         # No apt pinning for cis
         assert [] == m_add_pin.call_args_list
+        assert 1 == m_setup_apt_proxy.call_count
         assert subp_apt_cmds == m_subp.call_args_list
         assert 1 == m_should_reboot.call_count
         expected_stdout = (
             "Updating package lists\n"
             "Installing CIS Audit packages\n"
             "CIS Audit enabled\n"
+            "Visit {} to learn how to use CIS\n".format(CIS_DOCS_URL)
         )
         assert (expected_stdout, "") == capsys.readouterr()
