@@ -21,62 +21,193 @@ def entitlement(request, entitlement_factory):
 
 class TestESMRepoPinPriority:
     @pytest.mark.parametrize(
-        "esm_class, series, repo_pin_priority",
+        "series, is_active_esm, repo_pin_priority",
         (
-            (ESMInfraEntitlement, "trusty", "never"),
-            (ESMInfraEntitlement, "xenial", None),
-            (ESMInfraEntitlement, "bionic", None),
-            (ESMInfraEntitlement, "focal", None),
-            (ESMAppsEntitlement, "trusty", None),
-            (ESMAppsEntitlement, "xenial", None),
-            (ESMAppsEntitlement, "bionic", None),
-            (ESMAppsEntitlement, "focal", None),
+            ("trusty", True, "never"),
+            ("xenial", True, "never"),
+            ("bionic", False, None),
+            ("focal", False, None),
         ),
     )
+    @mock.patch("uaclient.util.is_active_esm")
     @mock.patch("uaclient.entitlements.esm.util.get_platform_info")
-    def test_esm_infra_repo_pin_priority_never_on_trusty(
-        self, m_get_platform_info, esm_class, series, repo_pin_priority
+    def test_esm_infra_repo_pin_priority_never_on_active_esm(
+        self,
+        m_get_platform_info,
+        m_is_active_esm,
+        series,
+        is_active_esm,
+        repo_pin_priority,
     ):
-        """Repository pinning priority for ESMInfra will be 'never' on trusty.
+        """Repository pinning priority for ESMInfra is 'never' when active ESM
 
-        A pin priority of 'never' means we setup and advertize ESM Infra
-        packages without allowing them to be installed until someone attaches
+        A pin priority of 'never' means we advertize those service packages
+        without allowing them to be installed until someone attaches
         the machine to Ubuntu Advantage. This is only done for ESM Infra
-        on Trusty. We won't want/need to advertize ESM packages on Xenial or
-        later. Since we don't advertize ESM Apps on any series,
-        repo_pin_priority is None on all series.
+        when the release is EOL and supports ESM. We won't want/need to
+        advertize ESM Infra packages on releases that are not EOL.
         """
+        m_is_active_esm.return_value = is_active_esm
         m_get_platform_info.return_value = {"series": series}
-        inst = esm_class({})
+        inst = ESMInfraEntitlement({})
         assert repo_pin_priority == inst.repo_pin_priority
+        assert [mock.call(series)] == m_is_active_esm.call_args_list
+
+    @pytest.mark.parametrize(
+        "series, is_lts, is_beta, cfg_allow_beta, repo_pin_priority",
+        (
+            # When esm non-beta pin it on non-trusty
+            ("trusty", True, False, None, None),
+            ("xenial", True, False, None, "never"),
+            ("bionic", True, False, None, "never"),
+            ("focal", True, False, None, "never"),
+            # when ESM beta don't pin
+            ("trusty", True, True, None, None),
+            ("xenial", True, True, None, None),
+            ("bionic", True, True, None, None),
+            ("focal", True, True, None, None),
+        ),
+    )
+    @mock.patch("uaclient.util.is_lts")
+    @mock.patch("uaclient.entitlements.esm.util.get_platform_info")
+    @mock.patch("uaclient.entitlements.UAConfig")
+    def test_esm_apps_repo_pin_priority_never_on_on_lts(
+        self,
+        m_cfg,
+        m_get_platform_info,
+        m_is_lts,
+        series,
+        is_lts,
+        is_beta,
+        cfg_allow_beta,
+        repo_pin_priority,
+        FakeConfig,
+    ):
+        """Repository pinning priority for ESMApps is 'never' when on LTS.
+
+        A pin priority of 'never' means we advertize those service packages
+        without allowing them to be installed until someone attaches
+        the machine to Ubuntu Advantage. This is only done for ESM Apps
+        when the release is an Ubuntu LTS release. We won't want/need to
+        advertize ESM Apps packages on non-LTS releases or if ESM Apps is beta.
+        """
+        m_is_lts.return_value = is_lts
+        m_get_platform_info.return_value = {"series": series}
+        cfg = FakeConfig.for_attached_machine()
+        if cfg_allow_beta:
+            cfg.override_features({"allow_beta": cfg_allow_beta})
+        m_cfg.return_value = cfg
+
+        inst = ESMAppsEntitlement(cfg)
+        with mock.patch.object(ESMAppsEntitlement, "is_beta", is_beta):
+            assert repo_pin_priority == inst.repo_pin_priority
+
+        is_lts_calls = []
+        if series != "trusty":
+            if cfg_allow_beta or not is_beta:
+                is_lts_calls = [mock.call(series)]
+        assert is_lts_calls == m_is_lts.call_args_list
 
 
 class TestESMDisableAptAuthOnly:
     @pytest.mark.parametrize(
-        "esm_class, series, disable_apt_auth_only",
+        "series, is_active_esm, disable_apt_auth_only",
         (
-            (ESMInfraEntitlement, "trusty", True),
-            (ESMInfraEntitlement, "xenial", False),
-            (ESMInfraEntitlement, "bionic", False),
-            (ESMInfraEntitlement, "focal", False),
-            (ESMAppsEntitlement, "trusty", False),
-            (ESMAppsEntitlement, "xenial", False),
-            (ESMAppsEntitlement, "bionic", False),
-            (ESMAppsEntitlement, "focal", False),
+            ("trusty", True, True),
+            ("xenial", True, True),
+            ("bionic", False, False),
+            ("focal", False, False),
         ),
     )
+    @mock.patch("uaclient.util.is_active_esm")
     @mock.patch("uaclient.entitlements.esm.util.get_platform_info")
-    def test_esm_infra_disable_apt_auth_only_is_true_on_trusty(
-        self, m_get_platform_info, esm_class, series, disable_apt_auth_only
+    def test_esm_infra_disable_apt_auth_only_is_true_when_active_esm(
+        self,
+        m_get_platform_info,
+        m_is_active_esm,
+        series,
+        is_active_esm,
+        disable_apt_auth_only,
     ):
+        m_is_active_esm.return_value = is_active_esm
         m_get_platform_info.return_value = {"series": series}
-        inst = esm_class({})
+        inst = ESMInfraEntitlement({})
         assert disable_apt_auth_only is inst.disable_apt_auth_only
+        assert [mock.call(series)] == m_is_active_esm.call_args_list
+
+    @pytest.mark.parametrize(
+        "series, is_lts, is_beta, cfg_allow_beta, disable_apt_auth_only",
+        (
+            ("trusty", True, True, None, False),
+            ("trusty", True, False, True, False),  # trusty always false
+            ("xenial", True, True, None, False),  # is_beta disables
+            ("xenial", True, False, False, True),  # not beta service succeeds
+            ("xenial", True, True, True, True),  # cfg allow_true overrides
+            ("bionic", True, True, None, False),
+            ("focal", True, True, None, False),
+            ("groovy", False, False, True, False),  # not is_lts fails
+        ),
+    )
+    @mock.patch("uaclient.util.is_lts")
+    @mock.patch("uaclient.entitlements.esm.util.get_platform_info")
+    @mock.patch("uaclient.entitlements.UAConfig")
+    def test_esm_apps_disable_apt_auth_only_is_true_on_lts(
+        self,
+        m_cfg,
+        m_get_platform_info,
+        m_is_lts,
+        series,
+        is_lts,
+        is_beta,
+        cfg_allow_beta,
+        disable_apt_auth_only,
+        FakeConfig,
+    ):
+        m_is_lts.return_value = is_lts
+        m_get_platform_info.return_value = {"series": series}
+        cfg = FakeConfig.for_attached_machine()
+        if cfg_allow_beta:
+            cfg.override_features({"allow_beta": cfg_allow_beta})
+        m_cfg.return_value = cfg
+
+        inst = ESMAppsEntitlement(cfg)
+        with mock.patch.object(ESMAppsEntitlement, "is_beta", is_beta):
+            print(is_beta, cfg_allow_beta)
+            print(inst.valid_service)
+            assert disable_apt_auth_only is inst.disable_apt_auth_only
+        is_lts_calls = []
+        if series != "trusty":
+            if cfg_allow_beta or not is_beta:
+                is_lts_calls = [mock.call(series)]
+        assert is_lts_calls == m_is_lts.call_args_list
 
 
+@mock.patch("uaclient.util.validate_proxy", side_effect=lambda x, y, z: y)
+@mock.patch("uaclient.entitlements.esm.update_ua_messages")
+@mock.patch("uaclient.apt.setup_apt_proxy")
 class TestESMInfraEntitlementEnable:
-    def test_enable_configures_apt_sources_and_auth_files(self, entitlement):
+    @pytest.mark.parametrize(
+        "esm_cls", [ESMAppsEntitlement, ESMInfraEntitlement]
+    )
+    def test_enable_configures_apt_sources_and_auth_files(
+        self,
+        m_setup_apt_proxy,
+        update_ua_messages,
+        m_validate_proxy,
+        esm_cls,
+        entitlement_factory,
+    ):
         """When entitled, configure apt repo auth token, pinning and url."""
+        entitlement = entitlement_factory(
+            esm_cls,
+            cfg_extension={
+                "ua_config": {
+                    "apt_http_proxy": "apt_http_proxy_value",
+                    "apt_https_proxy": "apt_https_proxy_value",
+                }
+            },
+            suites=["trusty"],
+        )
         patched_packages = ["a", "b"]
         original_exists = os.path.exists
 
@@ -94,6 +225,7 @@ class TestESMInfraEntitlementEnable:
             m_add_apt = stack.enter_context(
                 mock.patch("uaclient.apt.add_auth_apt_repo")
             )
+            stack.enter_context(mock.patch("uaclient.util.is_active_esm"))
             m_add_pinning = stack.enter_context(
                 mock.patch("uaclient.apt.add_ppa_pinning")
             )
@@ -121,9 +253,9 @@ class TestESMInfraEntitlementEnable:
                 mock.patch.object(type(entitlement), "packages", m_packages)
             )
 
-            m_can_enable.return_value = True
+            m_can_enable.return_value = (True, None)
 
-            assert True is entitlement.enable()
+            assert (True, None) == entitlement.enable()
 
         add_apt_calls = [
             mock.call(
@@ -153,7 +285,13 @@ class TestESMInfraEntitlementEnable:
             install_cmd,
         ]
 
-        assert [mock.call(silent=mock.ANY)] == m_can_enable.call_args_list
+        assert [mock.call()] == m_can_enable.call_args_list
+        assert [
+            mock.call(
+                http_proxy="apt_http_proxy_value",
+                https_proxy="apt_https_proxy_value",
+            )
+        ] == m_setup_apt_proxy.call_args_list
         assert add_apt_calls == m_add_apt.call_args_list
         assert 0 == m_add_pinning.call_count
         assert subp_calls == m_subp.call_args_list
@@ -166,9 +304,17 @@ class TestESMInfraEntitlementEnable:
         else:
             unlink_calls = []  # esm-apps doesn't write an apt pref file
         assert unlink_calls == m_unlink.call_args_list
+        assert [
+            mock.call(entitlement.cfg)
+        ] == update_ua_messages.call_args_list
 
     def test_enable_cleans_up_apt_sources_and_auth_files_on_error(
-        self, entitlement, caplog_text
+        self,
+        _m_setup_apt_proxy,
+        _update_ua_messages,
+        m_validate_proxy,
+        entitlement,
+        caplog_text,
     ):
         """When setup_apt_config fails, cleanup any apt artifacts."""
         original_exists = os.path.exists
@@ -197,6 +343,7 @@ class TestESMInfraEntitlementEnable:
             m_add_pinning = stack.enter_context(
                 mock.patch("uaclient.apt.add_ppa_pinning")
             )
+            stack.enter_context(mock.patch("uaclient.util.is_active_esm"))
             m_subp = stack.enter_context(
                 mock.patch("uaclient.util.subp", side_effect=fake_subp)
             )
@@ -218,7 +365,7 @@ class TestESMInfraEntitlementEnable:
                 mock.patch("uaclient.apt.os.unlink")
             )
 
-            m_can_enable.return_value = True
+            m_can_enable.return_value = (True, None)
 
             with pytest.raises(exceptions.UserFacingError) as excinfo:
                 entitlement.enable()
@@ -245,7 +392,7 @@ class TestESMInfraEntitlementEnable:
 
         error_msg = "APT update failed. Another process is running APT."
         assert error_msg == excinfo.value.msg
-        assert [mock.call(silent=mock.ANY)] == m_can_enable.call_args_list
+        assert [mock.call()] == m_can_enable.call_args_list
         assert add_apt_calls == m_add_apt.call_args_list
         assert 0 == m_add_pinning.call_count
         assert subp_calls == m_subp.call_args_list
@@ -264,12 +411,13 @@ class TestESMInfraEntitlementEnable:
         ] == m_remove_apt_config.call_args_list
 
 
+@mock.patch("uaclient.entitlements.esm.update_ua_messages")
 class TestESMEntitlementDisable:
     @pytest.mark.parametrize("silent", [False, True])
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch(M_PATH + "can_disable", return_value=False)
     def test_disable_returns_false_on_can_disable_false_and_does_nothing(
-        self, m_can_disable, m_platform_info, silent
+        self, m_can_disable, m_platform_info, _update_ua_messages, silent
     ):
         """When can_disable is false disable returns false and noops."""
         entitlement = ESMInfraEntitlement({})
@@ -283,7 +431,7 @@ class TestESMEntitlementDisable:
         "uaclient.util.get_platform_info", return_value={"series": "trusty"}
     )
     def test_disable_on_can_disable_true_removes_apt_config(
-        self, _m_platform_info, entitlement, tmpdir
+        self, _m_platform_info, update_ua_messages, entitlement, tmpdir
     ):
         """When can_disable, disable removes apt configuration"""
 
@@ -293,3 +441,6 @@ class TestESMEntitlementDisable:
             ) as m_remove_apt_config:
                 assert entitlement.disable(True)
         assert [mock.call()] == m_remove_apt_config.call_args_list
+        assert [
+            mock.call(entitlement.cfg)
+        ] == update_ua_messages.call_args_list
