@@ -3,7 +3,6 @@
 import contextlib
 import copy
 import io
-import itertools
 import mock
 import os
 from functools import partial
@@ -42,6 +41,32 @@ def entitlement(fips_entitlement_factory):
 
 
 class TestFIPSEntitlementDefaults:
+    @pytest.mark.parametrize("series", (("xenial"), ("bionic"), ("focal")))
+    @mock.patch("uaclient.util.get_platform_info")
+    def test_condiotional_packages(
+        self, m_get_platform_info, series, entitlement
+    ):
+        """Test conditional package respect series restrictions"""
+        m_get_platform_info.return_value = {"series": series}
+
+        conditional_packages = entitlement.conditional_packages
+        if series == "focal":
+            assert [
+                "strongswan",
+                "strongswan-hmac",
+                "openssh-client",
+                "openssh-server",
+            ] == conditional_packages
+        else:
+            assert [
+                "strongswan",
+                "strongswan-hmac",
+                "openssh-client",
+                "openssh-server",
+                "openssh-client-hmac",
+                "openssh-server-hmac",
+            ] == conditional_packages
+
     def test_default_repo_key_file(self, entitlement):
         """GPG keyring file is the same for both FIPS and FIPS with Updates"""
         assert entitlement.repo_key_file == "ubuntu-advantage-fips.gpg"
@@ -122,14 +147,24 @@ class TestFIPSEntitlementCanEnable:
                 "application_status",
                 return_value=(status.ApplicationStatus.DISABLED, ""),
             ):
-                assert True is entitlement.can_enable()
+                assert (True, None) == entitlement.can_enable()
         assert ("", "") == capsys.readouterr()
 
 
 class TestFIPSEntitlementEnable:
-    def test_enable_configures_apt_sources_and_auth_files(self, entitlement):
+    @mock.patch("uaclient.apt.setup_apt_proxy")
+    def test_enable_configures_apt_sources_and_auth_files(
+        self, m_setup_apt_proxy, entitlement
+    ):
         """When entitled, configure apt repo auth token, pinning and url."""
         patched_packages = ["a", "b"]
+        expected_conditional_packages = [
+            "openssh-server",
+            "openssh-server-hmac",
+            "strongswan",
+            "strongswan-hmac",
+        ]
+
         with contextlib.ExitStack() as stack:
             m_add_apt = stack.enter_context(
                 mock.patch("uaclient.apt.add_auth_apt_repo")
@@ -137,6 +172,12 @@ class TestFIPSEntitlementEnable:
             m_add_pinning = stack.enter_context(
                 mock.patch("uaclient.apt.add_ppa_pinning")
             )
+            m_installed_pkgs = stack.enter_context(
+                mock.patch(
+                    "uaclient.apt.get_installed_packages",
+                    return_value=["openssh-server", "strongswan"],
+                )
+            )
             m_subp = stack.enter_context(
                 mock.patch("uaclient.util.subp", return_value=("", ""))
             )
@@ -144,7 +185,7 @@ class TestFIPSEntitlementEnable:
                 mock.patch.object(entitlement, "can_enable")
             )
             stack.enter_context(
-                mock.patch(M_REPOPATH + "handle_message_operations")
+                mock.patch("uaclient.util.handle_message_operations")
             )
             stack.enter_context(
                 mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
@@ -157,9 +198,9 @@ class TestFIPSEntitlementEnable:
                 mock.patch.object(type(entitlement), "packages", m_packages)
             )
 
-            m_can_enable.return_value = True
+            m_can_enable.return_value = (True, None)
 
-            assert True is entitlement.enable()
+            assert (True, None) == entitlement.enable()
 
         repo_url = "http://{}".format(entitlement.name.upper())
         add_apt_calls = [
@@ -181,20 +222,43 @@ class TestFIPSEntitlementEnable:
                 1001,
             )
         ]
-        install_cmd = mock.call(
-            [
-                "apt-get",
-                "install",
-                "--assume-yes",
-                '-o Dpkg::Options::="--force-confdef"',
-                '-o Dpkg::Options::="--force-confold"',
-            ]
-            + patched_packages,
-            capture=True,
-            retry_sleeps=apt.APT_RETRIES,
-            env={"DEBIAN_FRONTEND": "noninteractive"},
+
+        install_cmd = []
+        install_cmd.append(
+            mock.call(
+                [
+                    "apt-get",
+                    "install",
+                    "--assume-yes",
+                    "--allow-downgrades",
+                    '-o Dpkg::Options::="--force-confdef"',
+                    '-o Dpkg::Options::="--force-confold"',
+                ]
+                + patched_packages,
+                capture=True,
+                retry_sleeps=apt.APT_RETRIES,
+                env={"DEBIAN_FRONTEND": "noninteractive"},
+            )
         )
 
+        for pkg in expected_conditional_packages:
+            install_cmd.append(
+                mock.call(
+                    [
+                        "apt-get",
+                        "install",
+                        "--assume-yes",
+                        "--allow-downgrades",
+                        '-o Dpkg::Options::="--force-confdef"',
+                        '-o Dpkg::Options::="--force-confold"',
+                        pkg,
+                    ],
+                    capture=True,
+                    retry_sleeps=apt.APT_RETRIES,
+                    env={"DEBIAN_FRONTEND": "noninteractive"},
+                )
+            )
+
         if isinstance(entitlement, FIPSEntitlement):
             subp_calls = [
                 mock.call(
@@ -213,12 +277,14 @@ class TestFIPSEntitlementEnable:
                     capture=True,
                     retry_sleeps=apt.APT_RETRIES,
                     env={},
-                ),
-                install_cmd,
+                )
             ]
         )
+        subp_calls += install_cmd
 
-        assert [mock.call(silent=mock.ANY)] == m_can_enable.call_args_list
+        assert [mock.call()] == m_can_enable.call_args_list
+        assert 1 == m_setup_apt_proxy.call_count
+        assert 1 == m_installed_pkgs.call_count
         assert add_apt_calls == m_add_apt.call_args_list
         assert apt_pinning_calls == m_add_pinning.call_args_list
         assert subp_calls == m_subp.call_args_list
@@ -226,30 +292,78 @@ class TestFIPSEntitlementEnable:
             ["", status.MESSAGE_FIPS_REBOOT_REQUIRED]
         ] == entitlement.cfg.read_cache("notices")
 
-    @mock.patch(
-        "uaclient.util.get_platform_info", return_value={"series": "xenial"}
+    @pytest.mark.parametrize(
+        "fips_common_enable_return_value, expected_remove_notice_calls",
+        [
+            (True, [mock.call("", status.MESSAGE_FIPS_INSTALL_OUT_OF_DATE)]),
+            (False, []),
+        ],
     )
-    def test_enable_returns_false_on_can_enable_false(
-        self, m_platform_info, entitlement
+    @mock.patch(
+        "uaclient.entitlements.fips.FIPSCommonEntitlement._perform_enable"
+    )
+    @mock.patch("uaclient.config.UAConfig.remove_notice")
+    def test_enable_removes_out_of_date_notice_on_success(
+        self,
+        m_remove_notice,
+        m_fips_common_enable,
+        fips_common_enable_return_value,
+        expected_remove_notice_calls,
+        entitlement_factory,
     ):
-        """When can_enable is false enable returns false and noops."""
-        with mock.patch.object(entitlement, "can_enable", return_value=False):
-            with mock.patch(M_REPOPATH + "handle_message_operations"):
-                assert False is entitlement.enable()
-        assert 0 == m_platform_info.call_count
+        m_fips_common_enable.return_value = fips_common_enable_return_value
+        fips_entitlement = entitlement_factory(FIPSEntitlement)
+        assert (
+            fips_common_enable_return_value
+            is fips_entitlement._perform_enable()
+        )
+        assert expected_remove_notice_calls == m_remove_notice.call_args_list
 
+    @pytest.mark.parametrize(
+        "repo_enable_return_value, expected_remove_notice_calls",
+        [
+            (
+                True,
+                [mock.call("", status.NOTICE_WRONG_FIPS_METAPACKAGE_ON_CLOUD)],
+            ),
+            (False, []),
+        ],
+    )
+    @mock.patch("uaclient.entitlements.repo.RepoEntitlement._perform_enable")
+    @mock.patch("uaclient.config.UAConfig.remove_notice")
+    def test_enable_removes_wrong_met_notice_on_success(
+        self,
+        m_remove_notice,
+        m_repo_enable,
+        repo_enable_return_value,
+        expected_remove_notice_calls,
+        entitlement,
+    ):
+        m_repo_enable.return_value = repo_enable_return_value
+        assert repo_enable_return_value is entitlement._perform_enable()
+        assert (
+            expected_remove_notice_calls == m_remove_notice.call_args_list[:1]
+        )
+
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch("uaclient.apt.add_auth_apt_repo")
     @mock.patch(
         "uaclient.util.get_platform_info", return_value={"series": "xenial"}
     )
     def test_enable_returns_false_on_missing_suites_directive(
-        self, m_platform_info, m_add_apt, fips_entitlement_factory
+        self,
+        m_platform_info,
+        m_add_apt,
+        _m_setup_apt_proxy,
+        fips_entitlement_factory,
     ):
         """When directives do not contain suites returns false."""
         entitlement = fips_entitlement_factory(suites=[])
 
-        with mock.patch.object(entitlement, "can_enable", return_value=True):
-            with mock.patch(M_REPOPATH + "handle_message_operations"):
+        with mock.patch.object(
+            entitlement, "can_enable", return_value=(True, None)
+        ):
+            with mock.patch("uaclient.util.handle_message_operations"):
                 with pytest.raises(exceptions.UserFacingError) as excinfo:
                     entitlement.enable()
         error_msg = "Empty {} apt suites directive from {}".format(
@@ -258,7 +372,10 @@ class TestFIPSEntitlementEnable:
         assert error_msg == excinfo.value.msg
         assert 0 == m_add_apt.call_count
 
-    def test_enable_errors_on_repo_pin_but_invalid_origin(self, entitlement):
+    @mock.patch("uaclient.apt.setup_apt_proxy")
+    def test_enable_errors_on_repo_pin_but_invalid_origin(
+        self, _m_setup_apt_proxy, entitlement
+    ):
         """Error when no valid origin is provided on a pinned entitlemnt."""
         entitlement.origin = None  # invalid value
 
@@ -269,9 +386,13 @@ class TestFIPSEntitlementEnable:
             m_add_pinning = stack.enter_context(
                 mock.patch("uaclient.apt.add_ppa_pinning")
             )
-            stack.enter_context(mock.patch.object(entitlement, "can_enable"))
             stack.enter_context(
-                mock.patch(M_REPOPATH + "handle_message_operations")
+                mock.patch.object(
+                    entitlement, "can_enable", return_value=(True, None)
+                )
+            )
+            stack.enter_context(
+                mock.patch("uaclient.util.handle_message_operations")
             )
             m_remove_apt_config = stack.enter_context(
                 mock.patch.object(entitlement, "remove_apt_config")
@@ -293,7 +414,10 @@ class TestFIPSEntitlementEnable:
         assert 0 == m_add_pinning.call_count
         assert 0 == m_remove_apt_config.call_count
 
-    def test_failure_to_install_removes_apt_auth(self, entitlement, tmpdir):
+    @mock.patch("uaclient.apt.setup_apt_proxy")
+    def test_failure_to_install_removes_apt_auth(
+        self, _m_setup_apt_proxy, entitlement, tmpdir
+    ):
 
         authfile = tmpdir.join("90ubuntu-advantage")
         authfile.write("")
@@ -308,10 +432,12 @@ class TestFIPSEntitlementEnable:
                 mock.patch("uaclient.util.subp", side_effect=fake_subp)
             )
             stack.enter_context(
-                mock.patch.object(entitlement, "can_enable", return_value=True)
+                mock.patch.object(
+                    entitlement, "can_enable", return_value=(True, None)
+                )
             )
             stack.enter_context(
-                mock.patch(M_REPOPATH + "handle_message_operations")
+                mock.patch("uaclient.util.handle_message_operations")
             )
             stack.enter_context(
                 mock.patch.object(
@@ -339,7 +465,7 @@ class TestFIPSEntitlementEnable:
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch("uaclient.util.is_config_value_true", return_value=False)
     @mock.patch("uaclient.util.prompt_for_confirmation", return_value=False)
-    @mock.patch("uaclient.entitlements.repo.handle_message_operations")
+    @mock.patch("uaclient.util.handle_message_operations")
     @mock.patch("uaclient.util.is_container", return_value=False)
     def test_enable_fails_when_livepatch_service_is_enabled(
         self,
@@ -376,7 +502,7 @@ class TestFIPSEntitlementEnable:
         )
         assert expected_msg.strip() in fake_stdout.getvalue().strip()
 
-    @mock.patch("uaclient.entitlements.repo.handle_message_operations")
+    @mock.patch("uaclient.util.handle_message_operations")
     @mock.patch(
         M_LIVEPATCH_PATH + "application_status",
         return_value=((status.ApplicationStatus.DISABLED, "")),
@@ -404,16 +530,45 @@ class TestFIPSEntitlementEnable:
                     status.ApplicationStatus.ENABLED,
                     "",
                 )
-                fake_stdout = io.StringIO()
-                with contextlib.redirect_stdout(fake_stdout):
-                    fips_entitlement.enable()
+                result, reason = fips_entitlement.enable()
+                assert not result
+                expected_msg = (
+                    "Cannot enable FIPS when FIPS Updates is enabled."
+                )
+                assert expected_msg.strip() == reason.message.strip()
 
-        expected_msg = "Cannot enable FIPS when FIPS Updates is enabled"
-        assert expected_msg.strip() == fake_stdout.getvalue().strip()
+    @mock.patch("uaclient.util.handle_message_operations")
+    @mock.patch(
+        M_LIVEPATCH_PATH + "application_status",
+        return_value=((status.ApplicationStatus.DISABLED, "")),
+    )
+    @mock.patch("uaclient.util.is_container", return_value=False)
+    def test_enable_fails_when_fips_updates_service_once_enabled(
+        self,
+        m_is_container,
+        m_livepatch,
+        m_handle_message_op,
+        entitlement_factory,
+    ):
+        m_handle_message_op.return_value = True
+        fips_entitlement = entitlement_factory(
+            FIPSEntitlement, services_once_enabled={"fips-updates": True}
+        )
+
+        with mock.patch.object(
+            fips_entitlement, "_allow_fips_on_cloud_instance"
+        ) as m_allow_fips_on_cloud:
+            m_allow_fips_on_cloud.return_value = True
+            result, reason = fips_entitlement.enable()
+            assert not result
+            expected_msg = (
+                "Cannot enable FIPS because FIPS Updates was once enabled."
+            )
+            assert expected_msg.strip() == reason.message.strip()
 
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch("uaclient.entitlements.fips.get_cloud_type")
-    @mock.patch("uaclient.entitlements.repo.handle_message_operations")
+    @mock.patch("uaclient.util.handle_message_operations")
     @mock.patch("uaclient.util.is_container", return_value=False)
     def test_enable_fails_when_on_xenial_cloud_instance(
         self,
@@ -432,18 +587,16 @@ class TestFIPSEntitlementEnable:
             "{}.application_status".format(base_path)
         ) as m_livepatch:
             m_livepatch.return_value = (status.ApplicationStatus.DISABLED, "")
-            fake_stdout = io.StringIO()
-            with contextlib.redirect_stdout(fake_stdout):
-                entitlement.enable()
-
-        expected_msg = """\
-        Ubuntu Xenial does not provide an Azure optimized FIPS kernel"""
-        assert expected_msg.strip() in fake_stdout.getvalue().strip()
+            result, reason = entitlement.enable()
+            assert not result
+            expected_msg = """\
+            Ubuntu Xenial does not provide an Azure optimized FIPS kernel"""
+            assert expected_msg.strip() in reason.message.strip()
 
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch("uaclient.util.is_config_value_true", return_value=False)
     @mock.patch("uaclient.entitlements.fips.get_cloud_type")
-    @mock.patch("uaclient.entitlements.repo.handle_message_operations")
+    @mock.patch("uaclient.util.handle_message_operations")
     @mock.patch("uaclient.util.is_container", return_value=False)
     def test_enable_fails_when_on_gcp_instance_with_default_fips(
         self,
@@ -471,13 +624,11 @@ class TestFIPSEntitlementEnable:
                 status.ApplicationStatus.DISABLED,
                 "",
             )
-            fake_stdout = io.StringIO()
-            with contextlib.redirect_stdout(fake_stdout):
-                entitlement.enable()
-
-        expected_msg = """\
-        Ubuntu Test does not provide a GCP optimized FIPS kernel"""
-        assert expected_msg.strip() in fake_stdout.getvalue().strip()
+            result, reason = entitlement.enable()
+            assert not result
+            expected_msg = """\
+            Ubuntu Test does not provide a GCP optimized FIPS kernel"""
+            assert expected_msg.strip() in reason.message.strip()
 
     @pytest.mark.parametrize("allow_xenial_fips_on_cloud", ((True), (False)))
     @pytest.mark.parametrize("cloud_id", (("aws"), ("gce"), ("azure"), (None)))
@@ -586,8 +737,23 @@ class TestFIPSEntitlementRemovePackages:
         else:
             assert 0 == m_subp.call_count
 
+    @mock.patch(M_GETPLATFORM, return_value={"series": "xenial"})
+    @mock.patch(M_PATH + "util.subp")
+    @mock.patch(M_PATH + "apt.get_installed_packages")
+    def test_remove_packages_output_message_when_fail(
+        self, m_get_installed_packages, m_subp, _m_get_platform, entitlement
+    ):
+        m_get_installed_packages.return_value = ["ubuntu-fips"]
+        m_subp.side_effect = util.ProcessExecutionError(cmd="test")
+        expected_msg = "Could not disable {}.".format(entitlement.title)
 
-@mock.patch(M_REPOPATH + "handle_message_operations", return_value=True)
+        with pytest.raises(exceptions.UserFacingError) as exc_info:
+            entitlement.remove_packages()
+
+        assert exc_info.value.msg.strip() == expected_msg
+
+
+@mock.patch("uaclient.util.handle_message_operations", return_value=True)
 @mock.patch("uaclient.util.should_reboot", return_value=True)
 @mock.patch(
     "uaclient.util.get_platform_info", return_value={"series": "xenial"}
@@ -712,32 +878,72 @@ class TestFIPSEntitlementApplicationStatus:
         assert expected_status == application_status
 
 
-def _fips_pkg_combinations():
-    """Construct all combinations of fips_packages and expected installs"""
-    fips_packages = {
-        "openssh-client": {"openssh-client-hmac"},
-        "openssh-server": {"openssh-server-hmac"},
-        "strongswan": {"strongswan-hmac"},
-    }
+class TestFipsEntitlementInstallPackages:
+    @mock.patch(M_PATH + "apt.run_apt_command")
+    def test_install_packages_fail_if_metapackage_not_installed(
+        self, m_run_apt, entitlement
+    ):
+        m_run_apt.side_effect = exceptions.UserFacingError("error")
+        with pytest.raises(exceptions.UserFacingError):
+            entitlement.install_packages()
 
-    items = [  # These are the items that we will combine together
-        (pkg_name, [pkg_name] + list(extra_pkgs))
-        for pkg_name, extra_pkgs in fips_packages.items()
-    ]
-    # This produces combinations in all possible combination lengths
-    combinations = itertools.chain.from_iterable(
-        itertools.combinations(items, n) for n in range(1, len(items))
-    )
-    ret = []
-    # This for loop flattens each combination together in to a single
-    # (installed_packages, expected_installs) item
-    for combination in combinations:
-        installed_packages, expected_installs = [], []
-        for pkg, installs in combination:
-            installed_packages.append(pkg)
-            expected_installs.extend(installs)
-        ret.append((installed_packages, expected_installs))
-    return ret
+    @mock.patch(M_PATH + "apt.get_installed_packages")
+    @mock.patch(M_PATH + "apt.run_apt_command")
+    def test_install_packages_dont_fail_if_conditional_pkgs_not_installed(
+        self, m_run_apt, m_installed_pkgs, fips_entitlement_factory
+    ):
+
+        conditional_pkgs = ["b", "c"]
+        m_installed_pkgs.return_value = conditional_pkgs
+        packages = ["a"]
+        entitlement = fips_entitlement_factory(additional_packages=packages)
+
+        m_run_apt.side_effect = [
+            True,
+            exceptions.UserFacingError("error"),
+            exceptions.UserFacingError("error"),
+        ]
+
+        fake_stdout = io.StringIO()
+        with contextlib.redirect_stdout(fake_stdout):
+            with mock.patch.object(
+                type(entitlement), "conditional_packages", conditional_pkgs
+            ):
+                entitlement.install_packages()
+
+        install_cmds = []
+        all_pkgs = packages + conditional_pkgs
+        for pkg in all_pkgs:
+            install_cmds.append(
+                mock.call(
+                    [
+                        "apt-get",
+                        "install",
+                        "--assume-yes",
+                        "--allow-downgrades",
+                        '-o Dpkg::Options::="--force-confdef"',
+                        '-o Dpkg::Options::="--force-confold"',
+                        pkg,
+                    ],
+                    "Could not enable {}.".format(entitlement.title),
+                    env={"DEBIAN_FRONTEND": "noninteractive"},
+                )
+            )
+
+        expected_msg = "\n".join(
+            [
+                "Installing {} packages".format(entitlement.title),
+                status.MESSAGE_FIPS_PACKAGE_NOT_AVAILABLE.format(
+                    service=entitlement.title, pkg="b"
+                ),
+                status.MESSAGE_FIPS_PACKAGE_NOT_AVAILABLE.format(
+                    service=entitlement.title, pkg="c"
+                ),
+            ]
+        )
+
+        assert install_cmds == m_run_apt.call_args_list
+        assert expected_msg.strip() in fake_stdout.getvalue().strip()
 
 
 class TestFipsSetupAPTConfig:
@@ -809,28 +1015,16 @@ class TestFipsEntitlementPackages:
             set(entitlement.packages)
         )
 
-    @pytest.mark.parametrize(
-        "installed_packages,expected_installs", _fips_pkg_combinations()
-    )
-    @mock.patch(M_PATH + "apt.get_installed_packages")
     @mock.patch("uaclient.util.get_platform_info")
     def test_currently_installed_packages_are_included_in_packages(
-        self,
-        m_platform_info,
-        m_get_installed_packages,
-        entitlement,
-        installed_packages,
-        expected_installs,
+        self, m_platform_info, entitlement
     ):
-        """If FIPS packages are already installed, upgrade them"""
-        m_get_installed_packages.return_value = list(installed_packages)
-
         # Do not trigger metapackage override by
         # _replace_metapackage_on_cloud_instance
-        m_platform_info.return_value = {"series": "test"}
+        # and xenial should not trigger that
+        m_platform_info.return_value = {"series": "xenial"}
 
-        full_expected_installs = FIPS_ADDITIONAL_PACKAGES + expected_installs
-        assert sorted(full_expected_installs) == sorted(entitlement.packages)
+        assert sorted(FIPS_ADDITIONAL_PACKAGES) == sorted(entitlement.packages)
 
     @mock.patch(M_PATH + "apt.get_installed_packages")
     @mock.patch("uaclient.util.get_platform_info")
@@ -858,7 +1052,18 @@ class TestFipsEntitlementPackages:
     @pytest.mark.parametrize(
         "series", (("trusty"), ("xenial"), ("bionic"), ("focal"))
     )
-    @pytest.mark.parametrize("cloud_id", (("azure"), ("aws"), ("gce"), (None)))
+    @pytest.mark.parametrize(
+        "cloud_id",
+        (
+            ("azure-china"),
+            ("aws-gov"),
+            ("aws-china"),
+            ("azure"),
+            ("aws"),
+            ("gce"),
+            (None),
+        ),
+    )
     @mock.patch("uaclient.util.is_config_value_true")
     @mock.patch(M_PATH + "get_cloud_type")
     @mock.patch("uaclient.util.get_platform_info")
@@ -888,10 +1093,12 @@ class TestFipsEntitlementPackages:
         if all(
             [
                 series == "bionic",
-                cloud_id in ("azure", "aws"),
+                cloud_id
+                in ("azure", "aws", "aws-china", "aws-gov", "azure-china"),
                 not cfg_disable_fips_metapckage_override,
             ]
         ):
+            cloud_id = cloud_id.split("-")[0]
             assert packages == [
                 "test1",
                 "ubuntu-{}-fips".format(cloud_id),
@@ -899,3 +1106,36 @@ class TestFipsEntitlementPackages:
             ]
         else:
             assert packages == additional_packages
+
+
+class TestFIPSUpdatesEntitlementEnable:
+    @pytest.mark.parametrize("enable_ret", ((True), (False)))
+    @mock.patch(
+        "uaclient.entitlements.fips.FIPSCommonEntitlement._perform_enable"
+    )
+    def test_fips_updates_enable_write_service_once_enable_file(
+        self, m_perform_enable, enable_ret, entitlement_factory
+    ):
+        m_perform_enable.return_value = enable_ret
+        m_write_cache = mock.MagicMock()
+        m_read_cache = mock.MagicMock()
+        m_read_cache.return_value = {}
+
+        cfg = mock.MagicMock()
+        cfg.read_cache = m_read_cache
+        cfg.write_cache = m_write_cache
+
+        fips_updates_ent = entitlement_factory(FIPSUpdatesEntitlement, cfg=cfg)
+        assert fips_updates_ent._perform_enable() == enable_ret
+
+        if enable_ret:
+            assert 1 == m_read_cache.call_count
+            assert 1 == m_write_cache.call_count
+            assert [
+                mock.call(
+                    key="services-once-enabled", content={"fips-updates": True}
+                )
+            ] == m_write_cache.call_args_list
+        else:
+            assert not m_read_cache.call_count
+            assert not m_write_cache.call_count
