@@ -22,7 +22,11 @@ from uaclient import exceptions
 from uaclient.entitlements.livepatch import (
     LivepatchEntitlement,
     process_config_directives,
+    configure_livepatch_proxy,
+    get_config_option_value,
+    unconfigure_livepatch_proxy,
 )
+from uaclient.snap import SNAP_CMD
 from uaclient.entitlements.tests.conftest import machine_token
 from uaclient import status
 from uaclient.status import ApplicationStatus, ContractStatus
@@ -67,6 +71,170 @@ def entitlement(livepatch_entitlement_factory):
     return livepatch_entitlement_factory()
 
 
+class TestConfigureLivepatchProxy:
+    @pytest.mark.parametrize(
+        "http_proxy,https_proxy,retry_sleeps",
+        (
+            ("http_proxy", "https_proxy", [1, 2]),
+            ("http_proxy", "", None),
+            ("", "https_proxy", [1, 2]),
+            ("http_proxy", None, [1, 2]),
+            (None, "https_proxy", None),
+            (None, None, [1, 2]),
+        ),
+    )
+    @mock.patch("uaclient.util.subp")
+    def test_configure_livepatch_proxy(
+        self, m_subp, http_proxy, https_proxy, retry_sleeps, capsys
+    ):
+        configure_livepatch_proxy(http_proxy, https_proxy, retry_sleeps)
+        expected_calls = []
+        if http_proxy:
+            expected_calls.append(
+                mock.call(
+                    [
+                        "canonical-livepatch",
+                        "config",
+                        "http-proxy={}".format(http_proxy),
+                    ],
+                    retry_sleeps=retry_sleeps,
+                )
+            )
+
+        if https_proxy:
+            expected_calls.append(
+                mock.call(
+                    [
+                        "canonical-livepatch",
+                        "config",
+                        "https-proxy={}".format(https_proxy),
+                    ],
+                    retry_sleeps=retry_sleeps,
+                )
+            )
+
+        assert m_subp.call_args_list == expected_calls
+
+        out, _ = capsys.readouterr()
+        if http_proxy or https_proxy:
+            assert out.strip() == status.MESSAGE_SETTING_SERVICE_PROXY.format(
+                service=LivepatchEntitlement.title
+            )
+
+    @pytest.mark.parametrize(
+        "key, subp_return_value, expected_ret",
+        [
+            ("http-proxy", ("nonsense", ""), None),
+            ("http-proxy", ("", "nonsense"), None),
+            (
+                "http-proxy",
+                (
+                    """\
+http-proxy: ""
+https-proxy: ""
+no-proxy: ""
+remote-server: https://livepatch.canonical.com
+ca-certs: ""
+check-interval: 60  # minutes""",
+                    "",
+                ),
+                None,
+            ),
+            (
+                "http-proxy",
+                (
+                    """\
+http-proxy: one
+https-proxy: two
+no-proxy: ""
+remote-server: https://livepatch.canonical.com
+ca-certs: ""
+check-interval: 60  # minutes""",
+                    "",
+                ),
+                "one",
+            ),
+            (
+                "https-proxy",
+                (
+                    """\
+http-proxy: one
+https-proxy: two
+no-proxy: ""
+remote-server: https://livepatch.canonical.com
+ca-certs: ""
+check-interval: 60  # minutes""",
+                    "",
+                ),
+                "two",
+            ),
+            (
+                "nonexistentkey",
+                (
+                    """\
+http-proxy: one
+https-proxy: two
+no-proxy: ""
+remote-server: https://livepatch.canonical.com
+ca-certs: ""
+check-interval: 60  # minutes""",
+                    "",
+                ),
+                None,
+            ),
+        ],
+    )
+    @mock.patch("uaclient.util.subp")
+    def test_get_config_option_value(
+        self, m_util_subp, key, subp_return_value, expected_ret
+    ):
+        m_util_subp.return_value = subp_return_value
+        ret = get_config_option_value(key)
+        assert ret == expected_ret
+        assert [
+            mock.call(["canonical-livepatch", "config"])
+        ] == m_util_subp.call_args_list
+
+
+class TestUnconfigureLivepatchProxy:
+    @pytest.mark.parametrize(
+        "livepatch_installed, protocol_type, retry_sleeps",
+        (
+            (True, "http", None),
+            (True, "https", [1]),
+            (True, "http", []),
+            (False, "http", None),
+        ),
+    )
+    @mock.patch("uaclient.util.which")
+    @mock.patch("uaclient.util.subp")
+    def test_unconfigure_livepatch_proxy(
+        self, subp, which, livepatch_installed, protocol_type, retry_sleeps
+    ):
+        if livepatch_installed:
+            which.return_value = "/snap/bin/canonical-livepatch"
+        else:
+            which.return_value = None
+        kwargs = {"protocol_type": protocol_type}
+        if retry_sleeps is not None:
+            kwargs["retry_sleeps"] = retry_sleeps
+        assert None is unconfigure_livepatch_proxy(**kwargs)
+        if livepatch_installed:
+            expected_calls = [
+                mock.call(
+                    [
+                        "canonical-livepatch",
+                        "config",
+                        protocol_type + "-proxy=",
+                    ],
+                    retry_sleeps=retry_sleeps,
+                )
+            ]
+        else:
+            expected_calls = []
+        assert expected_calls == subp.call_args_list
+
+
 class TestLivepatchContractStatus:
     def test_contract_status_entitled(self, entitlement):
         """The contract_status returns ENTITLED when entitled is True."""
@@ -202,12 +370,12 @@ class TestLivepatchEntitlementCanEnable:
             with mock.patch("uaclient.util.is_container") as m_container:
                 m_platform.return_value = supported_kernel
                 m_container.return_value = False
-                assert entitlement.can_enable()
+                assert (True, None) == entitlement.can_enable()
         assert ("", "") == capsys.readouterr()
         assert [mock.call()] == m_container.call_args_list
 
     def test_can_enable_false_on_unsupported_kernel_min_version(
-        self, _m_is_container, _m_livepatch_status, capsys, entitlement
+        self, _m_is_container, _m_livepatch_status, entitlement
     ):
         """"False when on a kernel less or equal to minKernelVersion."""
         unsupported_min_kernel = copy.deepcopy(dict(PLATFORM_INFO_SUPPORTED))
@@ -215,15 +383,17 @@ class TestLivepatchEntitlementCanEnable:
         with mock.patch("uaclient.util.get_platform_info") as m_platform:
             m_platform.return_value = unsupported_min_kernel
             entitlement = LivepatchEntitlement(entitlement.cfg)
-            assert not entitlement.can_enable()
-        msg = (
-            "Livepatch is not available for kernel 4.2.9-00-generic.\n"
-            "Minimum kernel version required: 4.4\n"
-        )
-        assert (msg, "") == capsys.readouterr()
+            result, reason = entitlement.can_enable()
+            assert False is result
+            assert status.CanEnableFailureReason.INAPPLICABLE == reason.reason
+            msg = (
+                "Livepatch is not available for kernel 4.2.9-00-generic.\n"
+                "Minimum kernel version required: 4.4."
+            )
+            assert msg == reason.message
 
     def test_can_enable_false_on_unsupported_kernel_flavor(
-        self, _m_is_container, _m_livepatch_status, capsys, entitlement
+        self, _m_is_container, _m_livepatch_status, entitlement
     ):
         """"When on an unsupported kernel, can_enable returns False."""
         unsupported_kernel = copy.deepcopy(dict(PLATFORM_INFO_SUPPORTED))
@@ -231,12 +401,14 @@ class TestLivepatchEntitlementCanEnable:
         with mock.patch("uaclient.util.get_platform_info") as m_platform:
             m_platform.return_value = unsupported_kernel
             entitlement = LivepatchEntitlement(entitlement.cfg)
-            assert not entitlement.can_enable()
-        msg = (
-            "Livepatch is not available for kernel 4.4.0-140-notgeneric.\n"
-            "Supported flavors are: generic, lowlatency\n"
-        )
-        assert (msg, "") == capsys.readouterr()
+            result, reason = entitlement.can_enable()
+            assert False is result
+            assert status.CanEnableFailureReason.INAPPLICABLE == reason.reason
+            msg = (
+                "Livepatch is not available for kernel 4.4.0-140-notgeneric.\n"
+                "Supported flavors are: generic, lowlatency."
+            )
+            assert msg == reason.message
 
     @pytest.mark.parametrize(
         "kernel_version,meets_min_version",
@@ -254,7 +426,6 @@ class TestLivepatchEntitlementCanEnable:
         _m_livepatch_status,
         kernel_version,
         meets_min_version,
-        capsys,
         entitlement,
     ):
         """"When on an unsupported kernel version, can_enable returns False."""
@@ -264,35 +435,40 @@ class TestLivepatchEntitlementCanEnable:
             m_platform.return_value = unsupported_kernel
             entitlement = LivepatchEntitlement(entitlement.cfg)
             if meets_min_version:
-                assert entitlement.can_enable()
+                assert (True, None) == entitlement.can_enable()
             else:
-                assert not entitlement.can_enable()
-        if meets_min_version:
-            msg = ""
-        else:
-            msg = (
-                "Livepatch is not available for kernel {}.\n"
-                "Minimum kernel version required: 4.4\n".format(kernel_version)
-            )
-        assert (msg, "") == capsys.readouterr()
+                result, reason = entitlement.can_enable()
+                assert False is result
+                assert (
+                    status.CanEnableFailureReason.INAPPLICABLE == reason.reason
+                )
+                msg = (
+                    "Livepatch is not available for kernel {}.\n"
+                    "Minimum kernel version required: 4.4.".format(
+                        kernel_version
+                    )
+                )
+                assert msg == reason.message
 
     def test_can_enable_false_on_unsupported_architecture(
-        self, _m_is_container, _m_livepatch_status, capsys, entitlement
+        self, _m_is_container, _m_livepatch_status, entitlement
     ):
         """"When on an unsupported architecture, can_enable returns False."""
         unsupported_kernel = copy.deepcopy(dict(PLATFORM_INFO_SUPPORTED))
         unsupported_kernel["arch"] = "ppc64le"
         with mock.patch("uaclient.util.get_platform_info") as m_platform:
             m_platform.return_value = unsupported_kernel
-            assert not entitlement.can_enable()
-        msg = (
-            "Livepatch is not available for platform ppc64le.\n"
-            "Supported platforms are: x86_64\n"
-        )
-        assert (msg, "") == capsys.readouterr()
+            result, reason = entitlement.can_enable()
+            assert False is result
+            assert status.CanEnableFailureReason.INAPPLICABLE == reason.reason
+            msg = (
+                "Livepatch is not available for platform ppc64le.\n"
+                "Supported platforms are: x86_64."
+            )
+            assert msg == reason.message
 
     def test_can_enable_false_on_containers(
-        self, m_is_container, _m_livepatch_status, capsys, entitlement
+        self, m_is_container, _m_livepatch_status, entitlement
     ):
         """When is_container is True, can_enable returns False."""
         unsupported_min_kernel = copy.deepcopy(dict(PLATFORM_INFO_SUPPORTED))
@@ -301,9 +477,11 @@ class TestLivepatchEntitlementCanEnable:
             m_platform.return_value = unsupported_min_kernel
             m_is_container.return_value = True
             entitlement = LivepatchEntitlement(entitlement.cfg)
-            assert not entitlement.can_enable()
-        msg = "Cannot install Livepatch on a container\n"
-        assert (msg, "") == capsys.readouterr()
+            result, reason = entitlement.can_enable()
+            assert False is result
+            assert status.CanEnableFailureReason.INAPPLICABLE == reason.reason
+            msg = "Cannot install Livepatch on a container."
+            assert msg == reason.message
 
 
 class TestLivepatchProcessContractDeltas:
@@ -407,6 +585,9 @@ class TestLivepatchProcessContractDeltas:
         assert setup_calls == m_setup_livepatch_config.call_args_list
 
 
+@mock.patch("uaclient.util.validate_proxy", side_effect=lambda x, y, z: y)
+@mock.patch("uaclient.snap.configure_snap_proxy")
+@mock.patch("uaclient.entitlements.livepatch.configure_livepatch_proxy")
 class TestLivepatchEntitlementEnable:
 
     mocks_apt_update = [
@@ -450,30 +631,6 @@ class TestLivepatchEntitlementEnable:
         ),
     ]
 
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=False)
-    def test_enable_false_when_can_enable_false(
-        self, m_can_enable, caplog_text, capsys, entitlement
-    ):
-        """When can_enable returns False enable returns False."""
-        assert not entitlement.enable()
-        assert "" == caplog_text()
-        assert ("", "") == capsys.readouterr()  # No additional prints
-        assert [mock.call(silent=mock.ANY)] == m_can_enable.call_args_list
-
-    @pytest.mark.parametrize("silent_if_inapplicable", (True, False, None))
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=False)
-    def test_enable_passes_silent_if_inapplicable_through(
-        self, m_can_enable, caplog_text, entitlement, silent_if_inapplicable
-    ):
-        """When can_enable returns False enable returns False."""
-        kwargs = {}
-        if silent_if_inapplicable is not None:
-            kwargs["silent_if_inapplicable"] = silent_if_inapplicable
-        entitlement.enable(**kwargs)
-
-        expected_call = mock.call(silent=bool(silent_if_inapplicable))
-        assert [expected_call] == m_can_enable.call_args_list
-
     @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
     @pytest.mark.parametrize("apt_update_success", (True, False))
     @mock.patch("uaclient.util.get_platform_info")
@@ -481,7 +638,9 @@ class TestLivepatchEntitlementEnable:
     @mock.patch("uaclient.apt.run_apt_command")
     @mock.patch("uaclient.util.which", return_value=False)
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=True)
+    @mock.patch(
+        M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
+    )
     def test_enable_installs_snapd_and_livepatch_snap_when_absent(
         self,
         m_can_enable,
@@ -490,6 +649,9 @@ class TestLivepatchEntitlementEnable:
         m_run_apt,
         m_subp,
         _m_get_platform_info,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
         capsys,
         caplog_text,
         entitlement,
@@ -525,10 +687,13 @@ class TestLivepatchEntitlementEnable:
         else:
             assert expected_log in caplog_text()
         expected_calls = [
-            mock.call("/snap/bin/canonical-livepatch"),
             mock.call("/usr/bin/snap"),
+            mock.call("/snap/bin/canonical-livepatch"),
         ]
         assert expected_calls == m_which.call_args_list
+        assert m_validate_proxy.call_count == 2
+        assert m_snap_proxy.call_count == 1
+        assert m_livepatch_proxy.call_count == 1
 
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch("uaclient.util.subp", return_value=("snapd", ""))
@@ -536,7 +701,9 @@ class TestLivepatchEntitlementEnable:
         "uaclient.util.which", side_effect=lambda cmd: cmd == "/usr/bin/snap"
     )
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=True)
+    @mock.patch(
+        M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
+    )
     def test_enable_installs_only_livepatch_snap_when_absent_but_snapd_present(
         self,
         m_can_enable,
@@ -544,6 +711,9 @@ class TestLivepatchEntitlementEnable:
         m_which,
         m_subp,
         _m_get_platform_info,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
         capsys,
         entitlement,
     ):
@@ -563,19 +733,33 @@ class TestLivepatchEntitlementEnable:
         )
         assert (msg, "") == capsys.readouterr()
         expected_calls = [
-            mock.call("/snap/bin/canonical-livepatch"),
             mock.call("/usr/bin/snap"),
+            mock.call("/snap/bin/canonical-livepatch"),
         ]
         assert expected_calls == m_which.call_args_list
+        assert m_validate_proxy.call_count == 2
+        assert m_snap_proxy.call_count == 1
+        assert m_livepatch_proxy.call_count == 1
 
     @mock.patch("uaclient.util.subp")
     @mock.patch(
         "uaclient.util.which", side_effect=lambda cmd: cmd == "/usr/bin/snap"
     )
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=True)
-    def test_enable_bails_if_snap_cmd_exists_but_snapd_pkg_not_installed(
-        self, m_can_enable, m_app_status, m_which, m_subp, capsys, entitlement
+    @mock.patch(
+        M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
+    )
+    def test_enable_fails_if_snap_cmd_exists_but_snapd_pkg_not_installed(
+        self,
+        m_can_enable,
+        m_app_status,
+        m_which,
+        m_subp,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
+        capsys,
+        entitlement,
     ):
         """Install canonical-livepatch snap when not present on the system."""
         m_app_status.return_value = status.ApplicationStatus.ENABLED, "enabled"
@@ -590,12 +774,18 @@ class TestLivepatchEntitlementEnable:
             " cannot enable {}".format(entitlement.title)
         )
         assert expected_msg == excinfo.value.msg
+        assert m_validate_proxy.call_count == 0
+        assert m_snap_proxy.call_count == 0
+        assert m_livepatch_proxy.call_count == 0
 
+    @mock.patch("uaclient.apt.get_installed_packages", return_value=["snapd"])
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch("uaclient.util.subp")
-    @mock.patch("uaclient.util.which", return_value="/found/livepatch")
+    @mock.patch("uaclient.util.which", side_effect=[True, True])
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=True)
+    @mock.patch(
+        M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
+    )
     def test_enable_does_not_install_livepatch_snap_when_present(
         self,
         m_can_enable,
@@ -603,6 +793,10 @@ class TestLivepatchEntitlementEnable:
         m_which,
         m_subp,
         _m_get_platform_info,
+        _m_get_installed_packages,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
         capsys,
         entitlement,
     ):
@@ -610,14 +804,38 @@ class TestLivepatchEntitlementEnable:
         application_status = status.ApplicationStatus.ENABLED
         m_app_status.return_value = application_status, "enabled"
         assert entitlement.enable()
-        assert self.mocks_config == m_subp.call_args_list
+        subp_calls = [
+            mock.call(
+                [SNAP_CMD, "wait", "system", "seed.loaded"], capture=True
+            ),
+            mock.call(
+                [
+                    "/snap/bin/canonical-livepatch",
+                    "config",
+                    "remote-server=https://alt.livepatch.com",
+                ],
+                capture=True,
+            ),
+            mock.call(["/snap/bin/canonical-livepatch", "disable"]),
+            mock.call(
+                ["/snap/bin/canonical-livepatch", "enable", "livepatch-token"],
+                capture=True,
+            ),
+        ]
+        assert subp_calls == m_subp.call_args_list
         assert ("Canonical livepatch enabled.\n", "") == capsys.readouterr()
+        assert m_validate_proxy.call_count == 2
+        assert m_snap_proxy.call_count == 1
+        assert m_livepatch_proxy.call_count == 1
 
+    @mock.patch("uaclient.apt.get_installed_packages", return_value=["snapd"])
     @mock.patch("uaclient.util.get_platform_info")
     @mock.patch("uaclient.util.subp")
-    @mock.patch("uaclient.util.which", return_value="/found/livepatch")
+    @mock.patch("uaclient.util.which", side_effect=[True, True])
     @mock.patch(M_PATH + "LivepatchEntitlement.application_status")
-    @mock.patch(M_PATH + "LivepatchEntitlement.can_enable", return_value=True)
+    @mock.patch(
+        M_PATH + "LivepatchEntitlement.can_enable", return_value=(True, None)
+    )
     def test_enable_does_not_disable_inactive_livepatch_snap_when_present(
         self,
         m_can_enable,
@@ -625,6 +843,10 @@ class TestLivepatchEntitlementEnable:
         m_which,
         m_subp,
         _m_get_platform_info,
+        _m_get_installed_packages,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
         capsys,
         entitlement,
     ):
@@ -633,6 +855,9 @@ class TestLivepatchEntitlementEnable:
         m_app_status.return_value = status.ApplicationStatus.DISABLED, "nope"
         assert entitlement.enable()
         subp_no_livepatch_disable = [
+            mock.call(
+                [SNAP_CMD, "wait", "system", "seed.loaded"], capture=True
+            ),
             mock.call(
                 [
                     "/snap/bin/canonical-livepatch",
@@ -648,6 +873,9 @@ class TestLivepatchEntitlementEnable:
         ]
         assert subp_no_livepatch_disable == m_subp.call_args_list
         assert ("Canonical livepatch enabled.\n", "") == capsys.readouterr()
+        assert m_validate_proxy.call_count == 2
+        assert m_snap_proxy.call_count == 1
+        assert m_livepatch_proxy.call_count == 1
 
     @pytest.mark.parametrize(
         "cls_name, cls_title",
@@ -656,19 +884,21 @@ class TestLivepatchEntitlementEnable:
             ("FIPSUpdatesEntitlement", "FIPS Updates"),
         ),
     )
-    @mock.patch("uaclient.entitlements.repo.handle_message_operations")
+    @mock.patch("uaclient.util.handle_message_operations")
     @mock.patch("uaclient.util.is_container", return_value=False)
     def test_enable_fails_when_blocking_service_is_enabled(
         self,
         m_is_container,
         m_handle_message_op,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
         cls_name,
         cls_title,
         entitlement,
     ):
         m_handle_message_op.return_value = True
 
-        fake_stdout = io.StringIO()
         with mock.patch(M_LIVEPATCH_STATUS, return_value=DISABLED_APP_STATUS):
             with mock.patch(
                 "uaclient.entitlements.fips.{}.application_status".format(
@@ -676,13 +906,118 @@ class TestLivepatchEntitlementEnable:
                 )
             ) as m_fips:
                 m_fips.return_value = (status.ApplicationStatus.ENABLED, "")
+                result, reason = entitlement.enable()
+                assert not result
+
+                msg = "Cannot enable Livepatch when {} is enabled.".format(
+                    cls_title
+                )
+                assert msg.strip() == reason.message.strip()
+
+        assert m_validate_proxy.call_count == 0
+        assert m_snap_proxy.call_count == 0
+        assert m_livepatch_proxy.call_count == 0
+
+    @pytest.mark.parametrize("caplog_text", [logging.WARN], indirect=True)
+    @mock.patch("uaclient.util.which")
+    @mock.patch("uaclient.apt.get_installed_packages")
+    @mock.patch("uaclient.util.subp")
+    def test_enable_alerts_user_that_snapd_does_not_wait_command(
+        self,
+        m_subp,
+        m_installed_pkgs,
+        m_which,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
+        entitlement,
+        capsys,
+        caplog_text,
+    ):
+        m_which.side_effect = [True, False]
+        m_installed_pkgs.return_value = ["snapd"]
+        stderr_msg = (
+            "error: Unknown command `wait'. Please specify one command of: "
+            "abort, ack, buy, change, changes, connect, create-user, disable,"
+            " disconnect, download, enable, find, help, install, interfaces, "
+            "known, list, login, logout, refresh, remove, run or try"
+        )
+
+        m_subp.side_effect = [
+            ProcessExecutionError(
+                cmd="snapd wait system seed.loaded",
+                exit_code=-1,
+                stdout="",
+                stderr=stderr_msg,
+            ),
+            True,
+        ]
+
+        fake_stdout = io.StringIO()
+
+        with mock.patch.object(entitlement, "can_enable") as m_can_enable:
+            m_can_enable.return_value = (True, None)
+            with mock.patch.object(
+                entitlement, "setup_livepatch_config"
+            ) as m_setup_livepatch:
                 with contextlib.redirect_stdout(fake_stdout):
                     entitlement.enable()
 
-        expected_msg = "Cannot enable Livepatch when {} is enabled".format(
-            cls_title
+                assert 1 == m_can_enable.call_count
+                assert 1 == m_setup_livepatch.call_count
+
+        assert (
+            "Installing canonical-livepatch snap"
+            in fake_stdout.getvalue().strip()
         )
-        assert expected_msg.strip() == fake_stdout.getvalue().strip()
+
+        for msg in status.MESSAGE_SNAPD_DOES_NOT_HAVE_WAIT_CMD.split("\n"):
+            assert msg in caplog_text()
+
+        assert m_validate_proxy.call_count == 2
+        assert m_snap_proxy.call_count == 1
+        assert m_livepatch_proxy.call_count == 1
+
+    @mock.patch("uaclient.util.which")
+    @mock.patch("uaclient.apt.get_installed_packages")
+    @mock.patch("uaclient.util.subp")
+    def test_enable_raise_exception_for_unexpected_error_on_snapd_wait(
+        self,
+        m_subp,
+        m_installed_pkgs,
+        m_which,
+        m_livepatch_proxy,
+        m_snap_proxy,
+        m_validate_proxy,
+        entitlement,
+    ):
+        m_which.side_effect = [False, True]
+        m_installed_pkgs.return_value = ["snapd"]
+        stderr_msg = "test error"
+
+        m_subp.side_effect = ProcessExecutionError(
+            cmd="snapd wait system seed.loaded",
+            exit_code=-1,
+            stdout="",
+            stderr=stderr_msg,
+        )
+
+        with mock.patch.object(entitlement, "can_enable") as m_can_enable:
+            m_can_enable.return_value = (True, None)
+            with mock.patch.object(
+                entitlement, "setup_livepatch_config"
+            ) as m_setup_livepatch:
+                with pytest.raises(ProcessExecutionError) as excinfo:
+                    entitlement.enable()
+
+            assert 1 == m_can_enable.call_count
+            assert 0 == m_setup_livepatch.call_count
+
+        expected_msg = "test error"
+        assert expected_msg in str(excinfo)
+        assert m_validate_proxy.call_count == 0
+        assert m_snap_proxy.call_count == 0
+        assert m_livepatch_proxy.call_count == 0
 
 
 class TestLivepatchApplicationStatus:
