@@ -5,11 +5,7 @@ import pytest
 from types import MappingProxyType
 
 from uaclient import apt
-from uaclient import config
-from uaclient.entitlements.repo import (
-    RepoEntitlement,
-    handle_message_operations,
-)
+from uaclient.entitlements.repo import RepoEntitlement
 from uaclient.entitlements.tests.conftest import machine_token
 from uaclient import exceptions
 from uaclient import status
@@ -392,23 +388,6 @@ class TestProcessContractDeltas:
 
 
 class TestRepoEnable:
-    @pytest.mark.parametrize("silent_if_inapplicable", (True, False, None))
-    @mock.patch.object(RepoTestEntitlement, "can_enable", return_value=False)
-    def test_enable_passes_silent_if_inapplicable_through(
-        self, m_can_enable, caplog_text, tmpdir, silent_if_inapplicable
-    ):
-        """When can_enable returns False enable returns False."""
-        cfg = config.UAConfig(cfg={"data_dir": tmpdir.strpath})
-        entitlement = RepoTestEntitlement(cfg)
-
-        kwargs = {}
-        if silent_if_inapplicable is not None:
-            kwargs["silent_if_inapplicable"] = silent_if_inapplicable
-        entitlement.enable(**kwargs)
-
-        expected_call = mock.call(silent=bool(silent_if_inapplicable))
-        assert [expected_call] == m_can_enable.call_args_list
-
     @pytest.mark.parametrize(
         "pre_enable_msg, output, can_enable_call_count",
         (
@@ -416,7 +395,11 @@ class TestRepoEnable:
             (["msg1", (lambda: True, {}), "msg2"], "msg1\nmsg2\n", 1),
         ),
     )
-    @mock.patch.object(RepoTestEntitlement, "can_enable", return_value=False)
+    @mock.patch.object(
+        RepoTestEntitlement,
+        "can_enable",
+        return_value=(False, status.CanEnableFailure(None)),
+    )
     def test_enable_can_exit_on_pre_enable_messaging_hooks(
         self,
         m_can_enable,
@@ -492,12 +475,15 @@ class TestRepoEnable:
     @pytest.mark.parametrize("should_reboot", (False, True))
     @pytest.mark.parametrize("with_pre_install_msg", (False, True))
     @pytest.mark.parametrize("packages", (["a"], [], None))
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "util.should_reboot")
     @mock.patch(M_PATH + "util.subp", return_value=("", ""))
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "os.path.exists", return_value=True)
     @mock.patch(M_PATH + "util.get_platform_info")
-    @mock.patch.object(RepoTestEntitlement, "can_enable", return_value=True)
+    @mock.patch.object(
+        RepoTestEntitlement, "can_enable", return_value=(True, None)
+    )
     def test_enable_calls_adds_apt_repo_and_calls_apt_update(
         self,
         m_can_enable,
@@ -506,6 +492,7 @@ class TestRepoEnable:
         m_apt_add,
         m_subp,
         m_should_reboot,
+        m_setup_apt_proxy,
         entitlement,
         capsys,
         caplog_text,
@@ -537,7 +524,7 @@ class TestRepoEnable:
             )
         ]
 
-        reboot_msg = "A reboot is required to complete install"
+        reboot_msg = "A reboot is required to complete install."
         expected_output = (
             "\n".join(
                 ["Updating package lists", "Repo Test Class enabled"]
@@ -598,12 +585,14 @@ class TestRepoEnable:
         ]
         assert add_apt_calls == m_apt_add.call_args_list
         assert 1 == m_should_reboot.call_count
+        assert 1 == m_setup_apt_proxy.call_count
         stdout, _ = capsys.readouterr()
         assert expected_output == stdout
 
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "util.subp")
     def test_failed_install_removes_apt_config_and_packages(
-        self, m_subp, entitlement
+        self, m_subp, _m_setup_apt_proxy, entitlement
     ):
         def fake_subp(args, *other_args, **kwargs):
             if "install" in args:
@@ -614,7 +603,7 @@ class TestRepoEnable:
         packages = ["fake_pkg", "and_another"]
         with mock.patch.object(entitlement, "setup_apt_config"):
             with mock.patch.object(
-                entitlement, "can_enable", return_value=True
+                entitlement, "can_enable", return_value=(True, None)
             ):
                 with mock.patch.object(
                     type(entitlement), "packages", packages
@@ -798,8 +787,15 @@ class TestSetupAptConfig:
             ),
         ),
     )
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     def test_missing_directives(
-        self, repo_cls, directives, exc_cls, err_msg, entitlement_factory
+        self,
+        _setup_apt_proxy,
+        repo_cls,
+        directives,
+        exc_cls,
+        err_msg,
+        entitlement_factory,
     ):
         """Raise an error when missing any required directives."""
         entitlement = entitlement_factory(repo_cls, directives=directives)
@@ -809,6 +805,7 @@ class TestSetupAptConfig:
         assert err_msg in str(excinfo.value)
 
     @pytest.mark.parametrize("enable_by_default", (True, False))
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_CONTRACT_PATH + "request_resource_machine_access")
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "apt.run_apt_command")
@@ -817,6 +814,7 @@ class TestSetupAptConfig:
         run_apt_command,
         add_auth_apt_repo,
         request_resource_machine_access,
+        _setup_apt_proxy,
         enable_by_default,
         entitlement_factory,
         caplog_text,
@@ -845,10 +843,33 @@ class TestSetupAptConfig:
                 mock.call("blah", "repotest")
             ] == request_resource_machine_access.call_args_list
 
+    @mock.patch("uaclient.apt.setup_apt_proxy")
+    @mock.patch("os.path.exists", return_value=False)
+    @mock.patch("uaclient.apt.add_auth_apt_repo")
+    @mock.patch("uaclient.apt.run_apt_command")
+    def test_calls_setup_apt_proxy(
+        self,
+        _m_run_apt_command,
+        _m_add_auth_repo,
+        _m_exists,
+        m_setup_apt_proxy,
+        entitlement,
+    ):
+        """Calls apt.setup_apt_proxy()"""
+        entitlement.setup_apt_config()
+        assert [
+            mock.call(http_proxy=None, https_proxy=None)
+        ] == m_setup_apt_proxy.call_args_list
+
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "apt.run_apt_command")
     def test_install_prerequisite_packages(
-        self, m_run_apt_command, m_add_auth_repo, entitlement
+        self,
+        m_run_apt_command,
+        m_add_auth_repo,
+        _m_setup_apt_proxy,
+        entitlement,
     ):
         """Install apt-transport-https and ca-certificates debs if absent.
 
@@ -874,9 +895,10 @@ class TestSetupAptConfig:
         )
         assert install_call in m_run_apt_command.call_args_list
 
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "util.get_platform_info")
     def test_setup_error_with_repo_pin_priority_and_missing_origin(
-        self, m_get_platform_info, entitlement_factory
+        self, m_get_platform_info, _setup_apt_proxy, entitlement_factory
     ):
         """Raise error when repo_pin_priority is set and origin is None."""
         m_get_platform_info.return_value = {"series": "xenial"}
@@ -890,6 +912,7 @@ class TestSetupAptConfig:
             in str(excinfo.value)
         )
 
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "os.unlink")
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "apt.run_apt_command")
@@ -900,6 +923,7 @@ class TestSetupAptConfig:
         m_run_apt_command,
         m_add_auth_repo,
         m_unlink,
+        _m_setup_apt_proxy,
         entitlement_factory,
     ):
         """When repo_pin_priority is never, disable repo in apt preferences."""
@@ -920,6 +944,7 @@ class TestSetupAptConfig:
             mock.call("/etc/apt/preferences.d/ubuntu-repotest")
         ] == m_unlink.call_args_list
 
+    @mock.patch("uaclient.apt.setup_apt_proxy")
     @mock.patch(M_PATH + "apt.add_auth_apt_repo")
     @mock.patch(M_PATH + "apt.run_apt_command")
     @mock.patch(M_PATH + "apt.add_ppa_pinning")
@@ -930,6 +955,7 @@ class TestSetupAptConfig:
         m_add_ppa_pinning,
         m_run_apt_command,
         m_add_auth_repo,
+        _m_setup_apt_proxy,
         entitlement_factory,
     ):
         """When repo_pin_priority is an int, set pin in apt preferences."""
@@ -1037,6 +1063,6 @@ class TestHandleMessageOperations:
     def test_handle_message_operations_for_strings_and_callables(
         self, msg_ops, retval, output, capsys
     ):
-        assert retval is handle_message_operations(msg_ops)
+        assert retval is util.handle_message_operations(msg_ops)
         out, _err = capsys.readouterr()
         assert output == out
