@@ -10,9 +10,13 @@ from textwrap import dedent
 import pytest
 
 from uaclient.apt import (
+    APT_CONFIG_PROXY_HTTP,
+    APT_CONFIG_PROXY_HTTPS,
+    APT_RETRIES,
     APT_KEYS_DIR,
     APT_AUTH_COMMENT,
     KEYRINGS_DIR,
+    APT_PROXY_CONF_FILE,
     add_apt_auth_conf_entry,
     add_auth_apt_repo,
     add_ppa_pinning,
@@ -24,6 +28,7 @@ from uaclient.apt import (
     remove_repo_from_apt_auth_file,
     assert_valid_apt_credentials,
     run_apt_command,
+    setup_apt_proxy,
 )
 from uaclient import apt, exceptions, util, status
 from uaclient.entitlements.tests.test_repo import RepoTestEntitlement
@@ -149,6 +154,7 @@ class TestValidAptCredentials:
                 expected_path,
             ],
             timeout=60,
+            retry_sleeps=APT_RETRIES,
         )
         assert [apt_helper_call] == m_subp.call_args_list
 
@@ -220,6 +226,7 @@ class TestValidAptCredentials:
                 expected_path,
             ],
             timeout=60,
+            retry_sleeps=APT_RETRIES,
         )
         assert [apt_helper_call] == m_subp.call_args_list
 
@@ -262,6 +269,7 @@ class TestValidAptCredentials:
                 expected_path,
             ],
             timeout=apt.APT_HELPER_TIMEOUT,
+            retry_sleeps=APT_RETRIES,
         )
         assert [apt_helper_call] == m_subp.call_args_list
 
@@ -909,5 +917,80 @@ class TestRunAptCommand:
                 error_msg=status.MESSAGE_APT_UPDATE_FAILED,
             )
 
-        expected_message = "\n".join(output_list)
+        expected_message = "\n".join(output_list) + "."
         assert expected_message == excinfo.value.msg
+
+
+class TestAptProxyConfig:
+    @pytest.mark.parametrize(
+        "kwargs, expected_remove_calls, expected_write_calls, expected_out",
+        [
+            ({}, [mock.call(APT_PROXY_CONF_FILE)], [], ""),
+            (
+                {"http_proxy": "mock_http_proxy"},
+                [],
+                [
+                    mock.call(
+                        APT_PROXY_CONF_FILE,
+                        status.MESSAGE_APT_PROXY_CONFIG_HEADER
+                        + APT_CONFIG_PROXY_HTTP.format(
+                            proxy_url="mock_http_proxy"
+                        ),
+                    )
+                ],
+                status.MESSAGE_SETTING_SERVICE_PROXY.format(service="APT"),
+            ),
+            (
+                {"https_proxy": "mock_https_proxy"},
+                [],
+                [
+                    mock.call(
+                        APT_PROXY_CONF_FILE,
+                        status.MESSAGE_APT_PROXY_CONFIG_HEADER
+                        + APT_CONFIG_PROXY_HTTPS.format(
+                            proxy_url="mock_https_proxy"
+                        ),
+                    )
+                ],
+                status.MESSAGE_SETTING_SERVICE_PROXY.format(service="APT"),
+            ),
+            (
+                {
+                    "http_proxy": "mock_http_proxy",
+                    "https_proxy": "mock_https_proxy",
+                },
+                [],
+                [
+                    mock.call(
+                        APT_PROXY_CONF_FILE,
+                        status.MESSAGE_APT_PROXY_CONFIG_HEADER
+                        + APT_CONFIG_PROXY_HTTP.format(
+                            proxy_url="mock_http_proxy"
+                        )
+                        + APT_CONFIG_PROXY_HTTPS.format(
+                            proxy_url="mock_https_proxy"
+                        ),
+                    )
+                ],
+                status.MESSAGE_SETTING_SERVICE_PROXY.format(service="APT"),
+            ),
+        ],
+    )
+    @mock.patch("uaclient.util.write_file")
+    @mock.patch("uaclient.util.remove_file")
+    def test_setup_apt_proxy_config(
+        self,
+        m_util_remove_file,
+        m_util_write_file,
+        kwargs,
+        expected_remove_calls,
+        expected_write_calls,
+        expected_out,
+        capsys,
+    ):
+        setup_apt_proxy(**kwargs)
+        assert expected_remove_calls == m_util_remove_file.call_args_list
+        assert expected_write_calls == m_util_write_file.call_args_list
+        out, err = capsys.readouterr()
+        assert expected_out == out.strip()
+        assert "" == err
