@@ -49,9 +49,9 @@ Client to manage Ubuntu Advantage services on a machine.
    (https://ubuntu.com/security/esm)
  - esm-infra: UA Infra: Extended Security Maintenance (ESM)
    (https://ubuntu.com/security/esm)
- - fips-updates: Uncertified security updates to FIPS modules
+ - fips-updates: NIST-certified core packages with priority security updates
    (https://ubuntu.com/security/certifications#fips)
- - fips: NIST-certified FIPS modules
+ - fips: NIST-certified core packages
    (https://ubuntu.com/security/certifications#fips)
  - livepatch: Canonical Livepatch service
    (https://ubuntu.com/security/livepatch)
@@ -61,11 +61,13 @@ Client to manage Ubuntu Advantage services on a machine.
 SERVICES_WRAPPED_HELP = textwrap.dedent(
     """
 Client to manage Ubuntu Advantage services on a machine.
+ - cis: Center for Internet Security Audit Tools
+   (https://ubuntu.com/security/certifications#cis)
  - esm-infra: UA Infra: Extended Security Maintenance (ESM)
    (https://ubuntu.com/security/esm)
- - fips-updates: Uncertified security updates to FIPS modules
+ - fips-updates: NIST-certified core packages with priority security updates
    (https://ubuntu.com/security/certifications#fips)
- - fips: NIST-certified FIPS modules
+ - fips: NIST-certified core packages
    (https://ubuntu.com/security/certifications#fips)
  - livepatch: Canonical Livepatch service
    (https://ubuntu.com/security/livepatch)
@@ -251,7 +253,7 @@ class TestCLIParser:
 
         status_msg = "enabled" if ent_msg == "yes" else "—"
         ufs_call_count = 1 if ent_msg == "yes" else 0
-        ent_name_call_count = 2 if ent_msg == "yes" else 1
+        ent_name_call_count = 3 if ent_msg == "yes" else 2
         is_beta_call_count = 1 if status_msg == "enabled" else 0
 
         expected_msgs = [
@@ -436,21 +438,6 @@ class TestMain:
     @pytest.mark.parametrize(
         "exception,expected_error_msg,expected_log",
         (
-            (
-                LockHeldError(
-                    pid="123",
-                    lock_request="ua reboot-cmds",
-                    lock_holder="ua auto-attach",
-                ),
-                "Unable to perform: ua reboot-cmds.\nOperation in progress:"
-                " ua auto-attach (pid:123)\n",
-                "LockHeldError",
-            ),
-            (
-                KeyboardInterrupt,
-                "Interrupt received; exiting.\n",
-                "KeyboardInterrupt",
-            ),
             (
                 TypeError("'NoneType' object is not subscriptable"),
                 status.MESSAGE_UNEXPECTED_ERROR + "\n",
@@ -491,11 +478,61 @@ class TestMain:
         assert "Traceback (most recent call last):" in error_log
         assert expected_log in error_log
 
+    @pytest.mark.parametrize(
+        "exception,expected_error_msg,expected_log",
+        (
+            (
+                KeyboardInterrupt,
+                "Interrupt received; exiting.\n",
+                "KeyboardInterrupt",
+            ),
+        ),
+    )
+    @mock.patch(M_PATH_UACONFIG + "delete_cache_key")
+    @mock.patch("uaclient.cli.setup_logging")
+    @mock.patch("uaclient.cli.get_parser")
+    def test_interrupt_errors_handled_gracefully(
+        self,
+        m_get_parser,
+        _m_setup_logging,
+        m_delete_cache_key,
+        capsys,
+        logging_sandbox,
+        caplog_text,
+        exception,
+        expected_error_msg,
+        expected_log,
+    ):
+        m_args = m_get_parser.return_value.parse_args.return_value
+        m_args.action.side_effect = exception
+
+        with pytest.raises(SystemExit) as excinfo:
+            with mock.patch("sys.argv", ["/usr/bin/ua", "subcmd"]):
+                main()
+        assert 0 == m_delete_cache_key.call_count
+
+        exc = excinfo.value
+        assert 1 == exc.code
+
+        out, err = capsys.readouterr()
+        assert "" == out
+        assert expected_error_msg == err
+        error_log = caplog_text()
+        assert expected_log in error_log
+
     @pytest.mark.parametrize(
         "exception,expected_exit_code",
         [
             (UserFacingError("You need to know about this."), 1),
             (AlreadyAttachedError(mock.MagicMock()), 0),
+            (
+                LockHeldError(
+                    pid="123",
+                    lock_request="ua reboot-cmds",
+                    lock_holder="ua auto-attach",
+                ),
+                1,
+            ),
         ],
     )
     @mock.patch("uaclient.cli.setup_logging")
@@ -533,7 +570,7 @@ class TestMain:
             [line.strip() for line in error_log.splitlines()]
         )
         assert expected_msg in error_log
-        assert "Traceback (most recent call last):" in error_log
+        assert "Traceback (most recent call last):" not in error_log
 
     @pytest.mark.parametrize(
         "error_url,expected_log",
@@ -580,6 +617,8 @@ class TestMain:
         assert "{}\n".format(status.MESSAGE_CONNECTIVITY_ERROR) == err
         error_log = caplog_text()
 
+        print(expected_log)
+        print(error_log)
         assert expected_log in error_log
         assert "Traceback (most recent call last):" in error_log
 
@@ -595,6 +634,22 @@ class TestMain:
 
         assert "['some', 'args']" in log
 
+    @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
+    @mock.patch("uaclient.cli.setup_logging")
+    @mock.patch("uaclient.cli.get_parser")
+    @mock.patch.dict(
+        os.environ, {"NOT_UA_ENV": "YES", "UA_FEATURES_WOW": "XYZ"}
+    )
+    def test_environment_is_logged(
+        self, _m_get_parser, _m_setup_logging, logging_sandbox, caplog_text
+    ):
+        main(["some", "args"])
+
+        log = caplog_text()
+
+        assert "UA_FEATURES_WOW=XYZ" in log
+        assert "NOT_UA_ENV=YES" not in log
+
     def test_argparse_errors_well_formatted(self, capsys):
         parser = get_parser()
         with mock.patch("sys.argv", ["ua", "enable"]):
