@@ -25,10 +25,13 @@ BASIC_MACHINE_TOKEN = {
     "availableResources": [],
     "machineToken": "non-empty-token",
     "machineTokenInfo": {
+        "machineId": "test_machine_id",
         "contractInfo": {
             "name": "mycontract",
             "id": "contract-1",
+            "createdAt": "2020-05-08T19:02:26Z",
             "resourceEntitlements": [],
+            "products": ["free"],
         },
         "accountInfo": {"id": "acct-1", "name": "accountName"},
     },
@@ -122,12 +125,18 @@ class TestActionAttach:
             ),
         ),
     )
+    @mock.patch("uaclient.util.should_reboot", return_value=False)
+    @mock.patch("uaclient.config.UAConfig.remove_notice")
     @mock.patch("uaclient.contract.get_available_resources")
+    @mock.patch("uaclient.config.update_ua_messages")
     @mock.patch(M_PATH + "contract.request_updated_contract")
     def test_status_updated_when_auto_enable_fails(
         self,
         request_updated_contract,
+        update_ua_messages,
         _m_get_available_resources,
+        _m_should_reboot,
+        _m_remove_notice,
         _m_get_uid,
         error_class,
         error_str,
@@ -157,13 +166,24 @@ class TestActionAttach:
         ), "Did not persist on disk status during attach failure"
         logs = caplog_text()
         assert expected_log in logs
+        assert [mock.call(cfg)] == update_ua_messages.call_args_list
 
+    @mock.patch("uaclient.util.should_reboot", return_value=False)
+    @mock.patch("uaclient.config.UAConfig.remove_notice")
+    @mock.patch("uaclient.config.update_ua_messages")
     @mock.patch(
         M_PATH + "contract.UAContractClient.request_contract_machine_attach"
     )
     @mock.patch(M_PATH + "action_status")
     def test_happy_path_with_token_arg(
-        self, action_status, contract_machine_attach, _m_getuid, FakeConfig
+        self,
+        action_status,
+        contract_machine_attach,
+        update_ua_messages,
+        _m_should_reboot,
+        _m_remove_notice,
+        _m_getuid,
+        FakeConfig,
     ):
         """A mock-heavy test for the happy path with the contract token arg"""
         # TODO: Improve this test with less general mocking and more
@@ -184,11 +204,22 @@ class TestActionAttach:
         assert 1 == action_status.call_count
         expected_calls = [mock.call(contract_token=token)]
         assert expected_calls == contract_machine_attach.call_args_list
+        assert [mock.call(cfg)] == update_ua_messages.call_args_list
 
     @pytest.mark.parametrize("auto_enable", (True, False))
+    @mock.patch("uaclient.util.should_reboot", return_value=False)
+    @mock.patch("uaclient.config.UAConfig.remove_notice")
     @mock.patch("uaclient.contract.get_available_resources")
+    @mock.patch("uaclient.config.update_ua_messages")
     def test_auto_enable_passed_through_to_request_updated_contract(
-        self, _m_getuid, _m_get_available_resources, auto_enable, FakeConfig
+        self,
+        update_ua_messages,
+        _m_get_available_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        _m_get_uid,
+        auto_enable,
+        FakeConfig,
     ):
         args = mock.MagicMock(auto_enable=auto_enable)
 
@@ -196,12 +227,14 @@ class TestActionAttach:
             cfg.write_cache("machine-token", BASIC_MACHINE_TOKEN)
             return True
 
+        cfg = FakeConfig()
         with mock.patch(M_PATH + "contract.request_updated_contract") as m_ruc:
             m_ruc.side_effect = fake_contract_updates
-            action_attach(args, FakeConfig())
+            action_attach(args, cfg)
 
         expected_call = mock.call(mock.ANY, mock.ANY, allow_enable=auto_enable)
         assert [expected_call] == m_ruc.call_args_list
+        assert [mock.call(cfg)] == update_ua_messages.call_args_list
 
 
 class TestParser:
