@@ -2,22 +2,46 @@ import mock
 import pytest
 import textwrap
 
-from uaclient.cli import action_disable
+from uaclient.cli import action_disable, main
 from uaclient import entitlements
 from uaclient import exceptions
 from uaclient import status
 
+
 ALL_SERVICE_MSG = "\n".join(
     textwrap.wrap(
-        "Try " + entitlements.ALL_ENTITLEMENTS_STR,
+        "Try " + ", ".join(entitlements.valid_services(allow_beta=True)) + ".",
         width=80,
         break_long_words=False,
     )
 )
 
+HELP_OUTPUT = textwrap.dedent(
+    """\
+usage: ua disable <service> [<service>] [flags]
+
+Disable an Ubuntu Advantage service.
+
+Arguments:
+  service       the name(s) of the Ubuntu Advantage services to disable One
+                of: cis, esm-infra, fips, fips-updates, livepatch
+
+Flags:
+  -h, --help    show this help message and exit
+  --assume-yes  do not prompt for confirmation before performing the disable
+"""
+)
+
 
 @mock.patch("uaclient.cli.os.getuid", return_value=0)
 class TestDisable:
+    def test_disable_help(self, _getuid, capsys):
+        with pytest.raises(SystemExit):
+            with mock.patch("sys.argv", ["/usr/bin/ua", "disable", "--help"]):
+                main()
+        out, _err = capsys.readouterr()
+        assert HELP_OUTPUT == out
+
     @pytest.mark.parametrize("service", [["testitlement"], ["ent1", "ent2"]])
     @pytest.mark.parametrize("assume_yes", (True, False))
     @pytest.mark.parametrize(
@@ -74,9 +98,11 @@ class TestDisable:
         assert len(entitlements_cls) == m_cfg.status.call_count
 
     @pytest.mark.parametrize("assume_yes", (True, False))
-    @mock.patch("uaclient.cli.entitlements")
+    @mock.patch(
+        "uaclient.entitlements.is_config_value_true", return_value=False
+    )
     def test_entitlements_not_found_disabled_and_enabled(
-        self, m_entitlements, _m_getuid, assume_yes, tmpdir
+        self, _m_is_config_value_true, _m_getuid, assume_yes, tmpdir
     ):
         expected_error_tmpl = status.MESSAGE_INVALID_SERVICE_OP_FAILURE_TMPL
         num_calls = 2
@@ -93,12 +119,7 @@ class TestDisable:
         m_ent3_obj = m_ent3_cls.return_value
         m_ent3_obj.disable.return_value = True
 
-        m_entitlements.ENTITLEMENT_CLASS_BY_NAME = {
-            "ent2": m_ent2_cls,
-            "ent3": m_ent3_cls,
-        }
-        m_entitlements.ALL_ENTITLEMENTS_STR = "ent2, ent3"
-        m_entitlements.RELEASED_ENTITLEMENTS_STR = "ent2, ent3"
+        m_ents_dict = {"ent2": m_ent2_cls, "ent3": m_ent3_cls}
 
         m_cfg = mock.Mock()
         m_cfg.check_lock_info.return_value = (-1, "")
@@ -107,12 +128,15 @@ class TestDisable:
         args_mock.service = ["ent1", "ent2", "ent3"]
         args_mock.assume_yes = assume_yes
 
-        with pytest.raises(exceptions.UserFacingError) as err:
-            action_disable(args_mock, m_cfg)
+        with mock.patch.object(
+            entitlements, "ENTITLEMENT_CLASS_BY_NAME", m_ents_dict
+        ):
+            with pytest.raises(exceptions.UserFacingError) as err:
+                action_disable(args_mock, m_cfg)
 
         assert (
             expected_error_tmpl.format(
-                operation="disable", name="ent1", service_msg="Try ent2, ent3"
+                operation="disable", name="ent1", service_msg="Try ent2, ent3."
             )
             == err.value.msg
         )
