@@ -2,21 +2,27 @@ import copy
 import datetime
 import itertools
 import json
+import logging
 import os
 import stat
+import yaml
 
 import mock
 import pytest
 
-from uaclient import entitlements, exceptions, status
+from uaclient import entitlements, exceptions, status, util, version
 from uaclient.config import (
     DataPath,
     DEFAULT_STATUS,
     PRIVATE_SUBDIR,
     UAConfig,
+    UA_CONFIGURABLE_KEYS,
+    VALID_UA_CONFIG_KEYS,
     parse_config,
     depth_first_merge_overlay_dict,
+    get_config_path,
 )
+from uaclient.defaults import CONFIG_DEFAULTS, DEFAULT_CONFIG_FILE
 from uaclient.entitlements import (
     ENTITLEMENT_CLASSES,
     ENTITLEMENT_CLASS_BY_NAME,
@@ -36,8 +42,8 @@ KNOWN_DATA_PATHS = (
 M_PATH = "uaclient.entitlements."
 
 DEFAULT_CFG_STATUS = {
-    "configStatus": DEFAULT_STATUS["configStatus"],
-    "configStatusDetails": DEFAULT_STATUS["configStatusDetails"],
+    "execution_status": DEFAULT_STATUS["execution_status"],
+    "execution_details": DEFAULT_STATUS["execution_details"],
 }
 
 ALL_RESOURCES_AVAILABLE = [
@@ -216,6 +222,125 @@ class TestDataPath:
         assert "/my/d/test_path" == cfg.data_path("test_path")
 
 
+CFG_BASE_CONTENT = """\
+# Ubuntu-Advantage client config file.
+# If you modify this file, run "ua refresh config" to ensure changes are
+# picked up by Ubuntu-Advantage client.
+
+contract_url: https://contracts.canonical.com
+data_dir: /var/lib/ubuntu-advantage
+log_file: /var/log/ubuntu-advantage.log
+log_level: debug
+security_url: https://ubuntu.com/security
+"""
+
+CFG_FEATURES_CONTENT = """\
+# Ubuntu-Advantage client config file.
+# If you modify this file, run "ua refresh config" to ensure changes are
+# picked up by Ubuntu-Advantage client.
+
+contract_url: https://contracts.canonical.com
+data_dir: /var/lib/ubuntu-advantage
+features:
+  extra_security_params:
+    hide: true
+  new: 2
+  show_beta: true
+log_file: /var/log/ubuntu-advantage.log
+log_level: debug
+security_url: https://ubuntu.com/security
+settings_overrides:
+  c: 1
+  d: 2
+"""
+
+UA_CFG_DICT = {
+    "ua_config": {
+        "apt_http_proxy": None,
+        "apt_https_proxy": None,
+        "http_proxy": None,
+        "https_proxy": None,
+    }
+}
+
+
+class TestUAConfigKeys:
+    @pytest.mark.parametrize("attr_name", UA_CONFIGURABLE_KEYS)
+    @mock.patch("uaclient.config.UAConfig.write_cfg")
+    def test_ua_configurable_keys_set_ua_config_dict(
+        self, write_cfg, attr_name, tmpdir
+    ):
+        """Getters and settings are available fo UA_CONFIGURABLE_KEYS."""
+        cfg = UAConfig({"data_dir": tmpdir.strpath})
+        assert None is getattr(cfg, attr_name)
+        setattr(cfg, attr_name, attr_name + "value")
+        assert attr_name + "value" == getattr(cfg, attr_name)
+        assert attr_name + "value" == cfg.cfg["ua_config"][attr_name]
+
+
+class TestWriteCfg:
+    @pytest.mark.parametrize("caplog_text", [logging.WARNING], indirect=True)
+    @pytest.mark.parametrize(
+        "orig_content, expected, warnings",
+        (
+            (
+                CFG_BASE_CONTENT,
+                CFG_BASE_CONTENT
+                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
+                [],
+            ),
+            (  # Yaml output is sorted alphabetically by key
+                "\n".join(sorted(CFG_BASE_CONTENT.splitlines(), reverse=True)),
+                CFG_BASE_CONTENT
+                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
+                [],
+            ),
+            # Any custom comments or unrecognized config keys are dropped
+            (
+                "unknown-keys-not-preserved: true\n# user comments are lost"
+                + CFG_BASE_CONTENT,
+                CFG_BASE_CONTENT
+                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
+                [
+                    "Ignoring invalid uaclient.conf key:"
+                    " unknown-keys-not-preserved=True"
+                ],
+            ),
+            # All features/settings_overrides ordered after ua_config
+            (
+                CFG_BASE_CONTENT
+                + "features:\n new: 2\n extra_security_params:\n  hide: true\n"
+                " show_beta: true\nsettings_overrides:\n d: 2\n c: 1\n",
+                CFG_FEATURES_CONTENT
+                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
+                [],
+            ),
+            (
+                "settings_overrides:\n c: 1\n d: 2\nfeatures:\n"
+                " show_beta: true\n new: 2\n extra_security_params:\n"
+                "  hide: true\nsettings_overrides:\n d: 2\n c: 1\n"
+                + CFG_BASE_CONTENT,
+                CFG_FEATURES_CONTENT
+                + yaml.dump(UA_CFG_DICT, default_flow_style=False),
+                [],
+            ),
+        ),
+    )
+    def test_write_cfg_reads_cfg_andpersists_structured_content_to_config_path(
+        self, orig_content, warnings, expected, caplog_text, tmpdir
+    ):
+        """write_cfg writes structured, ordered config YAML to config_path."""
+        orig_conf = tmpdir.join("orig_uaclient.conf")
+        orig_conf.write(orig_content)
+        cfg = UAConfig(cfg=parse_config(orig_conf.strpath))
+        out_conf = tmpdir.join("uaclient.conf")
+        cfg.write_cfg(out_conf.strpath)
+        assert expected == out_conf.read()
+        warn_logs = caplog_text()
+        for warning in warnings:
+            assert warning in warn_logs
+
+
 class TestWriteCache:
     @pytest.mark.parametrize(
         "key,clears_cache",
@@ -365,7 +490,11 @@ class TestReadCache:
             f.write('{"dt": "2019-07-25T14:35:51"}')
 
         actual = cfg.read_cache("dt_test")
-        assert {"dt": datetime.datetime(2019, 7, 25, 14, 35, 51)} == actual
+        assert {
+            "dt": datetime.datetime(
+                2019, 7, 25, 14, 35, 51, tzinfo=datetime.timezone.utc
+            )
+        } == actual
 
 
 class TestDeleteCacheKey:
@@ -525,12 +654,17 @@ class TestDeleteCache:
         )
 
 
+@mock.patch("uaclient.config.UAConfig.remove_notice")
+@mock.patch("uaclient.util.should_reboot", return_value=False)
 class TestStatus:
     esm_desc = ENTITLEMENT_CLASS_BY_NAME["esm-infra"].description
     cc_eal_desc = ENTITLEMENT_CLASS_BY_NAME["cc-eal"].description
 
-    def check_beta(self, cls, show_beta, uacfg=None):
+    def check_beta(self, cls, show_beta, uacfg=None, status=""):
         if not show_beta:
+            if status == "enabled":
+                return False
+
             if uacfg:
                 allow_beta = uacfg.cfg.get("features", {}).get(
                     "allow_beta", False
@@ -579,6 +713,8 @@ class TestStatus:
         self,
         _m_getuid,
         m_get_available_resources,
+        _m_should_reboot,
+        m_remove_notice,
         show_beta,
         expected_services,
         FakeConfig,
@@ -597,6 +733,17 @@ class TestStatus:
             m_get_cfg_status.return_value = DEFAULT_CFG_STATUS
             assert expected == cfg.status(show_beta=show_beta)
 
+            expected_calls = [
+                mock.call(
+                    "",
+                    status.MESSAGE_ENABLE_REBOOT_REQUIRED_TMPL.format(
+                        operation="fix operation"
+                    ),
+                )
+            ]
+
+            assert expected_calls == m_remove_notice.call_args_list
+
     @pytest.mark.parametrize("show_beta", (True, False))
     @pytest.mark.parametrize(
         "features_override", ((None), ({"allow_beta": True}))
@@ -636,6 +783,8 @@ class TestStatus:
         self,
         _m_getuid,
         m_get_avail_resources,
+        _m_should_reboot,
+        _m_remove_notice,
         avail_res,
         entitled_res,
         uf_entitled,
@@ -651,11 +800,21 @@ class TestStatus:
         token = {
             "availableResources": [],
             "machineTokenInfo": {
-                "accountInfo": {"id": "acct-1", "name": "test_account"},
+                "machineId": "test_machine_id",
+                "accountInfo": {
+                    "id": "acct-1",
+                    "name": "test_account",
+                    "createdAt": "2019-06-14T06:45:50Z",
+                    "externalAccountIDs": [{"IDs": ["id1"], "Origin": "AWS"}],
+                },
                 "contractInfo": {
                     "id": "cid",
                     "name": "test_contract",
+                    "createdAt": "2020-05-08T19:02:26Z",
+                    "effectiveFrom": "2000-05-08T19:02:26Z",
+                    "effectiveTo": "2040-05-08T19:02:26Z",
                     "resourceEntitlements": entitled_res,
+                    "products": ["free"],
                 },
             },
         }
@@ -688,8 +847,9 @@ class TestStatus:
                 "status": uf_status
                 if cls.name in resource_names
                 else default_status,
-                "statusDetails": mock.ANY,
+                "status_details": mock.ANY,
                 "description_override": None,
+                "available": mock.ANY,
             }
             for cls in entitlements.ENTITLEMENT_CLASSES
             if not self.check_beta(cls, show_beta, cfg)
@@ -697,12 +857,35 @@ class TestStatus:
         expected = copy.deepcopy(DEFAULT_STATUS)
         expected.update(
             {
-                "account-id": "acct-1",
-                "account": "test_account",
+                "version": version.get_version(features=cfg.features),
                 "attached": True,
+                "machine_id": "test_machine_id",
                 "services": expected_services,
-                "subscription": "test_contract",
-                "subscription-id": "cid",
+                "effective": datetime.datetime(
+                    2000, 5, 8, 19, 2, 26, tzinfo=datetime.timezone.utc
+                ),
+                "expires": datetime.datetime(
+                    2040, 5, 8, 19, 2, 26, tzinfo=datetime.timezone.utc
+                ),
+                "contract": {
+                    "name": "test_contract",
+                    "id": "cid",
+                    "created_at": datetime.datetime(
+                        2020, 5, 8, 19, 2, 26, tzinfo=datetime.timezone.utc
+                    ),
+                    "products": ["free"],
+                    "tech_support_level": "n/a",
+                },
+                "account": {
+                    "name": "test_account",
+                    "id": "acct-1",
+                    "created_at": datetime.datetime(
+                        2019, 6, 14, 6, 45, 50, tzinfo=datetime.timezone.utc
+                    ),
+                    "external_account_ids": [
+                        {"IDs": ["id1"], "Origin": "AWS"}
+                    ],
+                },
             }
         )
         with mock.patch(
@@ -724,14 +907,18 @@ class TestStatus:
     @mock.patch("uaclient.contract.get_available_resources")
     @mock.patch("uaclient.config.os.getuid")
     def test_nonroot_unattached_is_same_as_unattached_root(
-        self, m_getuid, m_get_available_resources, FakeConfig
+        self,
+        m_getuid,
+        m_get_available_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        FakeConfig,
     ):
         m_get_available_resources.return_value = [
             {"name": "esm-infra", "available": True}
         ]
         m_getuid.return_value = 1000
         cfg = FakeConfig()
-
         nonroot_status = cfg.status()
 
         m_getuid.return_value = 0
@@ -742,7 +929,13 @@ class TestStatus:
     @mock.patch("uaclient.contract.get_available_resources")
     @mock.patch("uaclient.config.os.getuid")
     def test_root_followed_by_nonroot(
-        self, m_getuid, m_get_available_resources, tmpdir, FakeConfig
+        self,
+        m_getuid,
+        m_get_available_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        tmpdir,
+        FakeConfig,
     ):
         """Ensure that non-root run after root returns data"""
         cfg = UAConfig({"data_dir": tmpdir.strpath})
@@ -768,7 +961,12 @@ class TestStatus:
     @mock.patch("uaclient.contract.get_available_resources", return_value=[])
     @mock.patch("uaclient.config.os.getuid", return_value=0)
     def test_cache_file_is_written_world_readable(
-        self, _m_getuid, _m_get_available_resources, tmpdir
+        self,
+        _m_getuid,
+        _m_get_available_resources,
+        _m_should_reboot,
+        m_remove_notice,
+        tmpdir,
     ):
         cfg = UAConfig({"data_dir": tmpdir.strpath})
         cfg.status()
@@ -777,6 +975,17 @@ class TestStatus:
             os.lstat(cfg.data_path("status-cache")).st_mode
         )
 
+        expected_calls = [
+            mock.call(
+                "",
+                status.MESSAGE_ENABLE_REBOOT_REQUIRED_TMPL.format(
+                    operation="fix operation"
+                ),
+            )
+        ]
+
+        assert expected_calls == m_remove_notice.call_args_list
+
     @pytest.mark.parametrize("show_beta", (True, False))
     @pytest.mark.parametrize(
         "features_override", ((None), ({"allow_beta": False}))
@@ -797,15 +1006,21 @@ class TestStatus:
     @mock.patch("uaclient.config.os.getuid", return_value=0)
     @mock.patch(M_PATH + "livepatch.LivepatchEntitlement.user_facing_status")
     @mock.patch(M_PATH + "livepatch.LivepatchEntitlement.contract_status")
+    @mock.patch(M_PATH + "esm.ESMAppsEntitlement.user_facing_status")
+    @mock.patch(M_PATH + "esm.ESMAppsEntitlement.contract_status")
     @mock.patch(M_PATH + "repo.RepoEntitlement.user_facing_status")
     @mock.patch(M_PATH + "repo.RepoEntitlement.contract_status")
     def test_attached_reports_contract_and_service_status(
         self,
         m_repo_contract_status,
         m_repo_uf_status,
+        m_esm_contract_status,
+        m_esm_uf_status,
         m_livepatch_contract_status,
         m_livepatch_uf_status,
         _m_getuid,
+        _m_should_reboot,
+        m_remove_notice,
         entitlements,
         features_override,
         show_beta,
@@ -824,14 +1039,27 @@ class TestStatus:
             status.UserFacingStatus.ACTIVE,
             "livepatch details",
         )
+        m_esm_contract_status.return_value = status.ContractStatus.ENTITLED
+        m_esm_uf_status.return_value = (
+            status.UserFacingStatus.ACTIVE,
+            "esm-apps details",
+        )
         token = {
             "availableResources": ALL_RESOURCES_AVAILABLE,
             "machineTokenInfo": {
-                "accountInfo": {"id": "1", "name": "accountname"},
+                "machineId": "test_machine_id",
+                "accountInfo": {
+                    "id": "1",
+                    "name": "accountname",
+                    "createdAt": "2019-06-14T06:45:50Z",
+                    "externalAccountIDs": [{"IDs": ["id1"], "Origin": "AWS"}],
+                },
                 "contractInfo": {
                     "id": "contract-1",
                     "name": "contractname",
+                    "createdAt": "2020-05-08T19:02:26Z",
                     "resourceEntitlements": entitlements,
+                    "products": ["free"],
                 },
             },
         }
@@ -847,32 +1075,53 @@ class TestStatus:
         expected = copy.deepcopy(DEFAULT_STATUS)
         expected.update(
             {
+                "version": version.get_version(features=cfg.features),
                 "attached": True,
-                "account": "accountname",
-                "account-id": "1",
-                "subscription": "contractname",
-                "subscription-id": "contract-1",
-                "techSupportLevel": support_level,
+                "machine_id": "test_machine_id",
+                "contract": {
+                    "name": "contractname",
+                    "id": "contract-1",
+                    "created_at": datetime.datetime(
+                        2020, 5, 8, 19, 2, 26, tzinfo=datetime.timezone.utc
+                    ),
+                    "products": ["free"],
+                    "tech_support_level": support_level,
+                },
+                "account": {
+                    "name": "accountname",
+                    "id": "1",
+                    "created_at": datetime.datetime(
+                        2019, 6, 14, 6, 45, 50, tzinfo=datetime.timezone.utc
+                    ),
+                    "external_account_ids": [
+                        {"IDs": ["id1"], "Origin": "AWS"}
+                    ],
+                },
             }
         )
         for cls in ENTITLEMENT_CLASSES:
-            if self.check_beta(cls, show_beta, cfg):
-                continue
-
             if cls.name == "livepatch":
                 expected_status = status.UserFacingStatus.ACTIVE.value
                 details = "livepatch details"
+            elif cls.name == "esm-apps":
+                expected_status = status.UserFacingStatus.ACTIVE.value
+                details = "esm-apps details"
             else:
                 expected_status = status.UserFacingStatus.INAPPLICABLE.value
                 details = "repo details"
+
+            if self.check_beta(cls, show_beta, cfg, expected_status):
+                continue
+
             expected["services"].append(
                 {
                     "name": cls.name,
                     "description": cls.description,
                     "entitled": status.ContractStatus.ENTITLED.value,
                     "status": expected_status,
-                    "statusDetails": details,
+                    "status_details": details,
                     "description_override": None,
+                    "available": mock.ANY,
                 }
             )
         with mock.patch(
@@ -880,23 +1129,43 @@ class TestStatus:
         ) as m_get_cfg_status:
             m_get_cfg_status.return_value = DEFAULT_CFG_STATUS
             assert expected == cfg.status(show_beta=show_beta)
-        assert len(ENTITLEMENT_CLASSES) - 1 == m_repo_uf_status.call_count
+
+        assert len(ENTITLEMENT_CLASSES) - 2 == m_repo_uf_status.call_count
         assert 1 == m_livepatch_uf_status.call_count
 
+        expected_calls = [
+            mock.call(
+                "",
+                status.MESSAGE_ENABLE_REBOOT_REQUIRED_TMPL.format(
+                    operation="fix operation"
+                ),
+            )
+        ]
+
+        assert expected_calls == m_remove_notice.call_args_list
+
     @mock.patch("uaclient.contract.get_available_resources")
     @mock.patch("uaclient.config.os.getuid")
     def test_expires_handled_appropriately(
-        self, m_getuid, _m_get_available_resources, FakeConfig
+        self,
+        m_getuid,
+        _m_get_available_resources,
+        _m_should_reboot,
+        _m_remove_notice,
+        FakeConfig,
     ):
         token = {
             "availableResources": ALL_RESOURCES_AVAILABLE,
             "machineTokenInfo": {
+                "machineId": "test_machine_id",
                 "accountInfo": {"id": "1", "name": "accountname"},
                 "contractInfo": {
                     "name": "contractname",
                     "id": "contract-1",
                     "effectiveTo": "2020-07-18T00:00:00Z",
+                    "createdAt": "2020-05-08T19:02:26Z",
                     "resourceEntitlements": [],
+                    "products": ["free"],
                 },
             },
         }
@@ -906,7 +1175,9 @@ class TestStatus:
 
         # Test that root's status works as expected (including the cache write)
         m_getuid.return_value = 0
-        expected_dt = datetime.datetime(2020, 7, 18, 0, 0, 0)
+        expected_dt = datetime.datetime(
+            2020, 7, 18, 0, 0, 0, tzinfo=datetime.timezone.utc
+        )
         assert expected_dt == cfg.status()["expires"]
 
         # Test that the read from the status cache work properly for non-root
@@ -916,27 +1187,31 @@ class TestStatus:
 
     @mock.patch("uaclient.config.os.getuid")
     def test_nonroot_user_uses_cache_and_updates_if_available(
-        self, m_getuid, tmpdir
+        self, m_getuid, _m_should_reboot, m_remove_notice, tmpdir
     ):
         m_getuid.return_value = 1000
 
-        status = {"pass": True}
+        expected_status = {"pass": True}
         cfg = UAConfig({"data_dir": tmpdir.strpath})
         cfg.write_cache("marker-reboot-cmds", "")  # To indicate a reboot reqd
-        cfg.write_cache("status-cache", status)
+        cfg.write_cache("status-cache", expected_status)
 
-        # Even non-root users can update configStatus details
+        # Even non-root users can update execution_status details
         details = MESSAGE_ENABLE_REBOOT_REQUIRED_TMPL.format(
             operation="configuration changes"
         )
-        status.update(
+        reboot_required = UserFacingConfigStatus.REBOOTREQUIRED.value
+        expected_status.update(
             {
-                "configStatus": UserFacingConfigStatus.REBOOTREQUIRED.value,
-                "configStatusDetails": details,
+                "execution_status": reboot_required,
+                "execution_details": details,
                 "notices": [],
+                "config_path": None,
+                "config": {"data_dir": mock.ANY},
             }
         )
-        assert status == cfg.status()
+
+        assert expected_status == cfg.status()
 
 
 ATTACHED_SERVICE_STATUS_PARAMETERS = [
@@ -989,6 +1264,145 @@ class TestAttachedServiceStatus:
         assert expected_status == ret["status"]
 
 
+class TestProcessConfig:
+    @pytest.mark.parametrize(
+        "http_proxy, https_proxy, snap_is_installed, snap_http_val, "
+        "snap_https_val, livepatch_enabled, livepatch_http_val, "
+        "livepatch_https_val, snap_livepatch_msg",
+        [
+            ("http", "https", False, None, None, False, None, None, ""),
+            ("http", "https", True, None, None, False, None, None, ""),
+            ("http", "https", False, None, None, True, None, None, ""),
+            ("http", "https", True, None, None, True, None, None, ""),
+            (None, None, True, None, None, True, None, None, ""),
+            (None, None, True, "one", None, True, None, None, "snap"),
+            (None, None, True, "one", "two", True, None, None, "snap"),
+            (
+                None,
+                None,
+                True,
+                "one",
+                "two",
+                True,
+                "three",
+                None,
+                "snap, livepatch",
+            ),
+            (
+                None,
+                None,
+                True,
+                "one",
+                "two",
+                True,
+                "three",
+                "four",
+                "snap, livepatch",
+            ),
+            (
+                None,
+                None,
+                False,
+                None,
+                None,
+                True,
+                "three",
+                "four",
+                "livepatch",
+            ),
+        ],
+    )
+    @mock.patch("uaclient.util.validate_proxy")
+    @mock.patch("uaclient.entitlements.livepatch.get_config_option_value")
+    @mock.patch("uaclient.entitlements.livepatch.configure_livepatch_proxy")
+    @mock.patch(
+        "uaclient.entitlements.livepatch.LivepatchEntitlement.application_status"  # noqa: E501
+    )
+    @mock.patch("uaclient.snap.get_config_option_value")
+    @mock.patch("uaclient.snap.configure_snap_proxy")
+    @mock.patch("uaclient.snap.is_installed")
+    @mock.patch("uaclient.apt.setup_apt_proxy")
+    def test_process_config(
+        self,
+        m_apt_configure_proxy,
+        m_snap_is_installed,
+        m_snap_configure_proxy,
+        m_snap_get_config_option,
+        m_livepatch_status,
+        m_livepatch_configure_proxy,
+        m_livepatch_get_config_option,
+        m_validate_proxy,
+        http_proxy,
+        https_proxy,
+        snap_is_installed,
+        snap_http_val,
+        snap_https_val,
+        livepatch_enabled,
+        livepatch_http_val,
+        livepatch_https_val,
+        snap_livepatch_msg,
+        capsys,
+    ):
+        m_snap_is_installed.return_value = snap_is_installed
+        m_snap_get_config_option.side_effect = [snap_http_val, snap_https_val]
+        m_livepatch_status.return_value = (
+            (status.ApplicationStatus.ENABLED, None)
+            if livepatch_enabled
+            else (None, None)
+        )
+        m_livepatch_get_config_option.side_effect = [
+            livepatch_http_val,
+            livepatch_https_val,
+        ]
+        cfg = UAConfig(
+            {
+                "ua_config": {
+                    "apt_http_proxy": "apt_http",
+                    "apt_https_proxy": "apt_https",
+                    "http_proxy": http_proxy,
+                    "https_proxy": https_proxy,
+                }
+            }
+        )
+
+        cfg.process_config()
+
+        assert [
+            mock.call("http", "apt_http", util.PROXY_VALIDATION_APT_HTTP_URL),
+            mock.call(
+                "https", "apt_https", util.PROXY_VALIDATION_APT_HTTPS_URL
+            ),
+            mock.call("http", http_proxy, util.PROXY_VALIDATION_SNAP_HTTP_URL),
+            mock.call(
+                "https", https_proxy, util.PROXY_VALIDATION_SNAP_HTTPS_URL
+            ),
+        ] == m_validate_proxy.call_args_list
+
+        assert [
+            mock.call("apt_http", "apt_https")
+        ] == m_apt_configure_proxy.call_args_list
+
+        if snap_is_installed:
+            assert [
+                mock.call(http_proxy, https_proxy)
+            ] == m_snap_configure_proxy.call_args_list
+
+        if livepatch_enabled:
+            assert [
+                mock.call(http_proxy, https_proxy)
+            ] == m_livepatch_configure_proxy.call_args_list
+
+        expected_out = ""
+        if snap_livepatch_msg:
+            expected_out = status.MESSAGE_PROXY_DETECTED_BUT_NOT_CONFIGURED.format(  # noqa: E501
+                services=snap_livepatch_msg
+            )
+
+        out, err = capsys.readouterr()
+        assert expected_out.strip() == out.strip()
+        assert "" == err
+
+
 class TestParseConfig:
     @mock.patch("uaclient.config.os.path.exists", return_value=False)
     def test_parse_config_uses_defaults_when_no_config_present(self, m_exists):
@@ -1002,63 +1416,168 @@ class TestParseConfig:
         assert expected_calls == m_exists.call_args_list
         expected_default_config = {
             "contract_url": "https://contracts.canonical.com",
+            "security_url": "https://ubuntu.com/security",
             "data_dir": "/var/lib/ubuntu-advantage",
             "log_file": "/var/log/ubuntu-advantage.log",
             "log_level": "INFO",
         }
         assert expected_default_config == config
 
-    @mock.patch("uaclient.config.os.path.exists", return_value=False)
-    def test_parse_config_scrubs_user_environ_values(self, m_exists):
-        user_values = {
-            "UA_CONTRACT_URL": "https://contract",
-            "ua_data_dir": "~/somedir",
-            "Ua_LoG_FiLe": "some.log",
-            "UA_LOG_LEVEL": "debug",
-        }
-        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
-            config = parse_config()
-        expanded_dir = os.path.expanduser("~")
-        expected_default_config = {
-            "contract_url": "https://contract",
-            "data_dir": "{}/somedir".format(expanded_dir),
-            "log_file": "some.log",
-            "log_level": "DEBUG",
-        }
-        assert expected_default_config == config
-
-    @mock.patch("uaclient.config.os.path.exists", return_value=False)
-    def test_parse_raises_errors_on_invalid_urls(self, m_exists):
-        user_values = {
-            "UA_CONTRACT_URL": "htp://contract"  # no acceptable url scheme
-        }
-        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
-            with pytest.raises(exceptions.UserFacingError) as excinfo:
-                parse_config()
-        expected_msg = "Invalid url in config. contract_url: htp://contract"
-        assert expected_msg == excinfo.value.msg
-
-
-class TestFeatures:
+    @pytest.mark.parametrize("caplog_text", [logging.WARNING], indirect=True)
     @pytest.mark.parametrize(
-        "cfg_features,expected",
+        "config_dict,warnings",
         (
-            ({}, {}),
-            (None, {}),
-            ({"feature1": "value1"}, {"feature1": "value1"}),
+            ({"contract_url": "http://abc", "security_url": "http:xyz"}, []),
             (
-                {"feature1": "value1", "feature2": False},
-                {"feature1": "value1", "feature2": False},
+                {"contract_urs": "http://abc", "security_url": "http:xyz"},
+                [
+                    "Ignoring invalid uaclient.conf key:"
+                    " contract_urs=http://abc\n"
+                ],
             ),
         ),
     )
-    def test_features_are_a_property_of_uaconfig(self, cfg_features, expected):
-        if cfg_features is None:
-            user_cfg = None
-        else:
-            user_cfg = {"features": cfg_features}
+    def test_parse_config_warns_and_ignores_invalid_config(
+        self, config_dict, warnings, caplog_text, tmpdir
+    ):
+        config_file = tmpdir.join("uaclient.conf")
+        config_file.write(yaml.dump(config_dict))
+        env_vars = {"UA_CONFIG_FILE": config_file.strpath}
+        with mock.patch.dict("uaclient.config.os.environ", values=env_vars):
+            cfg = parse_config(config_file.strpath)
+        expected = copy.deepcopy(CONFIG_DEFAULTS)
+        for key, value in config_dict.items():
+            if key in VALID_UA_CONFIG_KEYS:
+                expected[key] = config_dict[key]
+        warn_logs = caplog_text()
+        for warning in warnings:
+            assert warning in warn_logs
+        if not warnings:
+            assert "Ignoring invalid uaclient.conf key" not in warn_logs
+        assert expected == cfg
+
+    @pytest.mark.parametrize(
+        "envvar_name,envvar_val,field,expected_val",
+        [
+            # not on allowlist
+            (
+                "UA_CONTRACT_URL",
+                "https://contract",
+                "contract_url",
+                "https://contracts.canonical.com",
+            ),
+            # on allowlist
+            (
+                "UA_security_URL",
+                "https://security",
+                "security_url",
+                "https://security",
+            ),
+            (
+                "ua_data_dir",
+                "~/somedir",
+                "data_dir",
+                "{}/somedir".format(os.path.expanduser("~")),
+            ),
+            ("Ua_LoG_FiLe", "some.log", "log_file", "some.log"),
+            ("UA_LOG_LEVEL", "debug", "log_level", "debug"),
+        ],
+    )
+    @mock.patch("uaclient.config.os.path.exists", return_value=False)
+    def test_parse_config_scrubs_user_environ_values(
+        self, m_exists, envvar_name, envvar_val, field, expected_val
+    ):
+        user_values = {envvar_name: envvar_val}
+        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
+            config = parse_config()
+        assert expected_val == config[field]
+
+    @mock.patch("uaclient.config.os.path.exists", return_value=False)
+    def test_parse_config_scrubs_user_environ_values_features(self, m_exists):
+        user_values = {
+            "UA_FEATURES_X_Y_Z": "XYZ_VAL",
+            "UA_FEATURES_A_B_C": "ABC_VAL",
+        }
+        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
+            config = parse_config()
+        expected_config = {
+            "features": {"a_b_c": "ABC_VAL", "x_y_z": "XYZ_VAL"}
+        }
+        assert expected_config["features"] == config["features"]
+
+    @pytest.mark.parametrize(
+        "env_var,env_value", [("UA_SECURITY_URL", "ht://security")]
+    )
+    @mock.patch("uaclient.config.os.path.exists", return_value=False)
+    def test_parse_raises_errors_on_invalid_urls(
+        self, _m_exists, env_var, env_value
+    ):
+        user_values = {env_var: env_value}  # no acceptable url scheme
+        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
+            with pytest.raises(exceptions.UserFacingError) as excinfo:
+                parse_config()
+        expected_msg = "Invalid url in config. {}: {}".format(
+            env_var.replace("UA_", "").lower(), env_value
+        )
+        assert expected_msg == excinfo.value.msg
+
+    @mock.patch("uaclient.config.os.path.exists")
+    @mock.patch("uaclient.util.load_file")
+    def test_parse_reads_yaml_from_environ_values(
+        self, m_load_file, m_path_exists
+    ):
+        m_load_file.return_value = "test: true\nfoo: bar"
+        m_path_exists.side_effect = [False, False, True]
+
+        user_values = {"UA_FEATURES_TEST": "test.yaml"}
+        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
+            cfg = parse_config()
+
+        assert {"test": True, "foo": "bar"} == cfg["features"]["test"]
+
+    @mock.patch("uaclient.config.os.path.exists")
+    def test_parse_raise_exception_when_environ_yaml_file_does_not_exist(
+        self, m_path_exists
+    ):
+        m_path_exists.return_value = False
+        user_values = {"UA_FEATURES_TEST": "test.yaml"}
+        with mock.patch.dict("uaclient.config.os.environ", values=user_values):
+            with pytest.raises(exceptions.UserFacingError) as excinfo:
+                parse_config()
+
+        expected_msg = "Could not find yaml file: test.yaml"
+        assert expected_msg == excinfo.value.msg.strip()
+
+
+class TestFeatures:
+    @pytest.mark.parametrize("caplog_text", [logging.WARNING], indirect=True)
+    @pytest.mark.parametrize(
+        "cfg_features,expected, warnings",
+        (
+            ({}, {}, None),
+            (None, {}, None),
+            (
+                "badstring",
+                {},
+                "Unexpected uaclient.conf features value."
+                " Expected dict, but found badstring",
+            ),
+            ({"feature1": "value1"}, {"feature1": "value1"}, None),
+            (
+                {"feature1": "value1", "feature2": False},
+                {"feature1": "value1", "feature2": False},
+                None,
+            ),
+        ),
+    )
+    def test_features_are_a_property_of_uaconfig(
+        self, cfg_features, expected, warnings, caplog_text
+    ):
+        user_cfg = {"features": cfg_features}
         cfg = UAConfig(cfg=user_cfg)
         assert expected == cfg.features
+        if warnings:
+            assert warnings in caplog_text()
 
 
 class TestMachineTokenOverlay:
@@ -1239,3 +1758,25 @@ class TestDepthFirstMergeOverlayDict:
     ):
         depth_first_merge_overlay_dict(base_dict, overlay_dict)
         assert expected_dict == base_dict
+
+
+class TestGetConfigPath:
+    def test_get_config_path_from_env_var(self):
+        with mock.patch.dict(
+            "uaclient.config.os.environ", values={"UA_CONFIG_FILE": "test"}
+        ):
+            assert "test" == get_config_path()
+
+    @mock.patch("uaclient.config.os.path.join", return_value="test123")
+    @mock.patch("uaclient.config.os.path.exists", return_value=True)
+    def test_get_config_path_from_local_dir(self, _m_exists, _m_join):
+        with mock.patch.dict("uaclient.config.os.environ", values={}):
+            assert "test123" == get_config_path()
+            assert _m_join.call_count == 1
+            assert _m_exists.call_count == 1
+
+    @mock.patch("uaclient.config.os.path.exists", return_value=False)
+    def test_get_default_config_path(self, _m_exists):
+        with mock.patch.dict("uaclient.config.os.environ", values={}):
+            assert DEFAULT_CONFIG_FILE == get_config_path()
+            assert _m_exists.call_count == 1
