@@ -3,18 +3,30 @@ import pytest
 import string
 
 from uaclient import config
-from uaclient.status import format_tabular, TxtColor
+from uaclient.status import (
+    format_tabular,
+    TxtColor,
+    colorize_commands,
+    UserFacingStatus,
+)
 
 
-@pytest.fixture
-def status_dict_attached():
+@pytest.fixture(params=[True, False])
+def status_dict_attached(request):
     status = config.DEFAULT_STATUS.copy()
 
     # The following are required so we don't get an "unattached" error
     status["attached"] = True
-    status["account"] = "account"
-    status["subscription"] = "subscription"
     status["expires"] = "expires"
+    status["account"] = {"name": ""}
+    status["contract"] = {
+        "name": "",
+        "tech_support_level": UserFacingStatus.INAPPLICABLE.value,
+    }
+
+    if request.param:
+        status["account"]["name"] = "account"
+        status["contract"]["name"] = "subscription"
 
     return status
 
@@ -34,6 +46,54 @@ def status_dict_unattached():
     return status
 
 
+class TestColorizeCommands:
+    @pytest.mark.parametrize(
+        "commands,expected",
+        [
+            (
+                [
+                    ["apt", "update"],
+                    ["apt", "install", "--only-upgrade", "-y", "pkg"],
+                ],
+                TxtColor.DISABLEGREY
+                + "{ apt update && apt install --only-upgrade -y pkg }"
+                + TxtColor.ENDC,
+            ),
+            (
+                [
+                    ["apt", "update"],
+                    [
+                        "apt",
+                        "install",
+                        "--only-upgrade",
+                        "-y",
+                        "longpackagename1",
+                        "longpackagename2",
+                        "longpackagename3",
+                        "longpackagename4",
+                        "longpackagename5",
+                        "longpackagename6",
+                        "longpackagename7",
+                        "longpackagename8",
+                        "longpackagename9",
+                        "longpackagename10",
+                    ],
+                ],
+                TxtColor.DISABLEGREY
+                + "{\n"
+                + "  apt update && apt install --only-upgrade -y longpackagename1 \\\n"  # noqa: E501
+                + "  longpackagename2 longpackagename3 longpackagename4 longpackagename5 \\\n"  # noqa: E501
+                + "  longpackagename6 longpackagename7 longpackagename8 longpackagename9 \\\n"  # noqa: E501
+                + "  longpackagename10"
+                + "\n}"
+                + TxtColor.ENDC,
+            ),
+        ],
+    )
+    def test_colorize_commands(self, commands, expected):
+        assert colorize_commands(commands) == expected
+
+
 class TestFormatTabular:
     @pytest.mark.parametrize(
         "support_level,expected_colour,istty",
@@ -60,7 +120,7 @@ class TestFormatTabular:
         istty,
         status_dict_attached,
     ):
-        status_dict_attached["techSupportLevel"] = support_level
+        status_dict_attached["contract"]["tech_support_level"] = support_level
 
         m_isatty.return_value = istty
         tabular_output = format_tabular(status_dict_attached)
@@ -91,16 +151,8 @@ class TestFormatTabular:
     @pytest.mark.parametrize(
         "origin,expected_headers",
         [
-            ("free", ("Account", "Subscription")),
-            (
-                "not-free",
-                (
-                    "Account",
-                    "Subscription",
-                    "Valid until",
-                    "Technical support level",
-                ),
-            ),
+            ("free", ()),
+            ("not-free", ("Valid until", "Technical support level")),
         ],
     )
     def test_correct_header_keys_included(
@@ -108,6 +160,11 @@ class TestFormatTabular:
     ):
         status_dict_attached["origin"] = origin
 
+        if status_dict_attached["contract"].get("name"):
+            expected_headers = ("Subscription",) + expected_headers
+        if status_dict_attached["account"].get("name"):
+            expected_headers = ("Account",) + expected_headers
+
         tabular_output = format_tabular(status_dict_attached)
 
         headers = [
