@@ -3,8 +3,10 @@ import datetime
 import json
 import logging
 import posix
+import socket
 import subprocess
 import uuid
+import urllib
 
 import mock
 import pytest
@@ -119,35 +121,124 @@ class TestGetDictDeltas:
         assert expected == util.get_dict_deltas(orig_dict, new_dict)
 
 
+class TestIsLTS:
+    @pytest.mark.parametrize(
+        "series, supported_esm, expected",
+        (
+            ("trusty", "trusty\nxenial\nbionic\nfocal", True),
+            ("xenial", "trusty\nxenial\nbionic\nfocal", True),
+            ("groovy", "trusty\nxenial\nbionic\nfocal", False),
+        ),
+    )
+    @mock.patch("uaclient.util.subp")
+    def test_is_lts_if_distro_info_supported_esm(
+        self, subp, series, supported_esm, expected
+    ):
+        subp.return_value = supported_esm, ""
+        # Use __wrapped__ to avoid hitting the lru_cached value across tests
+        assert expected is util.is_lts.__wrapped__(series)
+        assert [
+            mock.call(["/usr/bin/ubuntu-distro-info", "--supported-esm"])
+        ] == subp.call_args_list
+
+
+class TestIsActiveESM:
+    @pytest.mark.parametrize(
+        "series, is_lts, days_until_esm,expected",
+        (
+            ("trusty", True, 0, True),
+            ("xenial", True, 1, False),
+            ("xenial", True, 0, True),
+            ("bionic", True, 1, False),
+            ("bionic", True, 0, True),
+            ("focal", True, 1, False),
+            ("focal", True, 0, True),
+            ("groovy", False, 0, False),
+        ),
+    )
+    @mock.patch("uaclient.util.subp")
+    @mock.patch("uaclient.util.is_lts")
+    def test_true_when_supported_esm_release_and_active(
+        self, util_is_lts, subp, series, is_lts, days_until_esm, expected
+    ):
+        """Return True when series is --suported-esm and days until esm < 1."""
+        util_is_lts.return_value = is_lts
+        subp.return_value = str(days_until_esm), ""
+
+        # Use __wrapped__ to avoid hitting the lru_cached value across tests
+        calls = []
+        if is_lts and series != "trusty":
+            calls.append(
+                mock.call(
+                    [
+                        "/usr/bin/ubuntu-distro-info",
+                        "--series",
+                        series,
+                        "-yeol",
+                    ]
+                )
+            )
+        assert expected is util.is_active_esm.__wrapped__(series)
+        assert calls == subp.call_args_list
+
+
 class TestIsContainer:
     @mock.patch("uaclient.util.subp")
     def test_true_systemd_detect_virt_success(self, m_subp):
         """Return True when systemd-detect virt exits success."""
-        m_subp.return_value = "", ""
+        util.is_container.cache_clear()
+        m_subp.side_effect = [
+            util.ProcessExecutionError(
+                "Failed running command 'ischroot' [exit(1)]"
+            ),
+            "",
+            "",
+        ]
         assert True is util.is_container()
         # Second call for lru_cache test
         util.is_container()
-        calls = [mock.call(["systemd-detect-virt", "--quiet", "--container"])]
+        calls = [
+            mock.call(["ischroot"]),
+            mock.call(["systemd-detect-virt", "--quiet", "--container"]),
+        ]
         assert calls == m_subp.call_args_list
 
     @mock.patch("uaclient.util.subp")
     def test_true_on_run_container_type(self, m_subp, tmpdir):
         """Return True when /run/container_type exists."""
-        m_subp.side_effect = OSError("No systemd-detect-virt utility")
+        util.is_container.cache_clear()
+        m_subp.side_effect = [
+            util.ProcessExecutionError(
+                "Failed running command 'ischroot' [exit(1)]"
+            ),
+            OSError("No systemd-detect-virt utility"),
+        ]
         tmpdir.join("container_type").write("")
 
         assert True is util.is_container(run_path=tmpdir.strpath)
-        calls = [mock.call(["systemd-detect-virt", "--quiet", "--container"])]
+        calls = [
+            mock.call(["ischroot"]),
+            mock.call(["systemd-detect-virt", "--quiet", "--container"]),
+        ]
         assert calls == m_subp.call_args_list
 
     @mock.patch("uaclient.util.subp")
     def test_true_on_run_systemd_container(self, m_subp, tmpdir):
         """Return True when /run/systemd/container exists."""
-        m_subp.side_effect = OSError("No systemd-detect-virt utility")
+        util.is_container.cache_clear()
+        m_subp.side_effect = [
+            util.ProcessExecutionError(
+                "Failed running command 'ischroot' [exit(1)]"
+            ),
+            OSError("No systemd-detect-virt utility"),
+        ]
         tmpdir.join("systemd/container").write("", ensure=True)
 
         assert True is util.is_container(run_path=tmpdir.strpath)
-        calls = [mock.call(["systemd-detect-virt", "--quiet", "--container"])]
+        calls = [
+            mock.call(["ischroot"]),
+            mock.call(["systemd-detect-virt", "--quiet", "--container"]),
+        ]
         assert calls == m_subp.call_args_list
 
     @mock.patch("uaclient.util.subp")
@@ -155,12 +246,22 @@ class TestIsContainer:
         self, m_subp, tmpdir
     ):
         """Return False when sytemd-detect-virt erros and no /run/* files."""
-        m_subp.side_effect = OSError("No systemd-detect-virt utility")
+        util.is_container.cache_clear()
+        m_subp.side_effect = [
+            util.ProcessExecutionError(
+                "Failed running command 'ischroot' [exit(1)]"
+            ),
+            OSError("No systemd-detect-virt utility"),
+        ]
+        tmpdir.join("systemd/container").write("", ensure=True)
 
         with mock.patch("uaclient.util.os.path.exists") as m_exists:
             m_exists.return_value = False
             assert False is util.is_container(run_path=tmpdir.strpath)
-        calls = [mock.call(["systemd-detect-virt", "--quiet", "--container"])]
+        calls = [
+            mock.call(["ischroot"]),
+            mock.call(["systemd-detect-virt", "--quiet", "--container"]),
+        ]
         assert calls == m_subp.call_args_list
         exists_calls = [
             mock.call(tmpdir.join("container_type").strpath),
@@ -168,6 +269,15 @@ class TestIsContainer:
         ]
         assert exists_calls == m_exists.call_args_list
 
+    @mock.patch("uaclient.util.subp")
+    def test_false_on_chroot_system(self, m_subp):
+        util.is_container.cache_clear()
+        m_subp.return_value = ("", "")
+        assert False is util.is_container()
+
+        calls = [mock.call(["ischroot"])]
+        assert calls == m_subp.call_args_list
+
 
 class TestSubp:
     def test_raise_error_on_timeout(self):
@@ -183,11 +293,12 @@ class TestSubp:
         with pytest.raises(util.ProcessExecutionError) as excinfo:
             util.subp(["ls", "--bogus"])
 
-        expected_error = (
-            "Failed running command 'ls --bogus' [exit(2)]."
-            " Message: ls: unrecognized option"
-        )
-        assert expected_error in str(excinfo.value)
+        expected_errors = [
+            "Failed running command 'ls --bogus' [exit(2)].",
+            "ls: unrecognized option '--bogus'",
+        ]
+        for msg in expected_errors:
+            assert msg in str(excinfo.value)
         assert 0 == m_sleep.call_count  # no retries
 
     @mock.patch("uaclient.util.time.sleep")
@@ -220,6 +331,25 @@ class TestSubp:
 
         assert expected_sleeps == sleeps
 
+    @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
+    @mock.patch("uaclient.util._subp")
+    @mock.patch("uaclient.util.time.sleep")
+    def test_retry_logs_remaining_retries(self, m_sleep, m_subp, caplog_text):
+        """When retry_sleeps given, use defined sleeps between each retry."""
+        sleeps = [1, 3, 0.4]
+        m_subp.side_effect = util.ProcessExecutionError("Funky apt %d error")
+        with pytest.raises(util.ProcessExecutionError):
+            util.subp(["apt", "dostuff"], retry_sleeps=sleeps)
+
+        logs = caplog_text()
+        expected_logs = [
+            "'Funky apt %d error'. Retrying 3 more times.",
+            "'Funky apt %d error'. Retrying 2 more times.",
+            "'Funky apt %d error'. Retrying 1 more times.",
+        ]
+        for log in expected_logs:
+            assert log in logs
+
 
 class TestParseOSRelease:
     @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
@@ -458,7 +588,105 @@ class TestIsServiceUrl:
 
 
 class TestReadurl:
-    def test_simple_call_with_url_works(self):
+    @pytest.mark.parametrize("caplog_text", [logging.DEBUG], indirect=True)
+    @pytest.mark.parametrize(
+        "headers,data,method,url,response,expected_logs",
+        (
+            (
+                {},
+                None,
+                None,
+                "http://some_url",
+                None,
+                [
+                    "URL [GET]: http://some_url, headers: {}, data: None",
+                    "URL [GET] response: http://some_url, headers: {},"
+                    " data: response\n",
+                ],
+            ),
+            # AWS PRO redacts IMDSv2 token
+            (
+                {
+                    "X-aws-ec2-metadata-token-ttl-seconds": "21600",
+                    "X-aws-ec2-metadata-token": "SEKRET",
+                },
+                None,
+                "PUT",
+                "http://169.254.169.254/latest/api/token",
+                b"SECRET==",
+                [
+                    "URL [PUT]: http://169.254.169.254/latest/api/token,"
+                    " headers: {'X-aws-ec2-metadata-token': '<REDACTED>',"
+                    " 'X-aws-ec2-metadata-token-ttl-seconds': '21600'}",
+                    "URL [PUT] response:"
+                    " http://169.254.169.254/latest/api/token, headers:"
+                    " {'X-aws-ec2-metadata-token': '<REDACTED>',"
+                    " 'X-aws-ec2-metadata-token-ttl-seconds': '21600'},"
+                    " data: <REDACTED>\n",
+                ],
+            ),
+            (
+                {"key1": "Bearcat", "Authorization": "Bearer SEKRET"},
+                b"{'token': 'HIDEME', 'tokenInfo': 'SHOWME'}",
+                None,
+                "http://some_url",
+                b"{'machineToken': 'HIDEME', 'machineTokenInfo': 'SHOWME'}",
+                [
+                    "URL [POST]: http://some_url, headers: {'Authorization':"
+                    " 'Bearer <REDACTED>', 'key1': 'Bearcat'}, data:"
+                    " {'token': '<REDACTED>', 'tokenInfo': 'SHOWME'}",
+                    "URL [POST] response: http://some_url, headers:"
+                    " {'Authorization': 'Bearer <REDACTED>', 'key1': 'Bearcat'"
+                    "}, data: {'machineToken': '<REDACTED>',"
+                    " 'machineTokenInfo': 'SHOWME'}",
+                ],
+            ),
+        ),
+    )
+    @mock.patch("uaclient.util.request.urlopen")
+    def test_readurl_redacts_call_and_response(
+        self,
+        urlopen,
+        headers,
+        data,
+        method,
+        url,
+        response,
+        expected_logs,
+        caplog_text,
+    ):
+        """Log and redact sensitive data from logs for url interactions."""
+
+        class FakeHTTPResponse:
+            def __init__(self, headers, content):
+                self.headers = headers
+                self._content = content
+
+            def read(self):
+                return self._content
+
+        if not response:
+            response = b"response"
+        urlopen.return_value = FakeHTTPResponse(
+            headers=headers, content=response
+        )
+        util.readurl(url, method=method, headers=headers, data=data)
+        logs = caplog_text()
+        for log in expected_logs:
+            assert log in logs
+
+    @pytest.mark.parametrize("timeout", (None, 1))
+    def test_simple_call_with_url_and_timeout_works(self, timeout):
+        with mock.patch("uaclient.util.request.urlopen") as m_urlopen:
+            if timeout:
+                util.readurl("http://some_url", timeout=timeout)
+            else:
+                util.readurl("http://some_url")
+        assert [
+            mock.call(mock.ANY, timeout=timeout)
+        ] == m_urlopen.call_args_list
+
+    def test_call_with_timeout(self):
         with mock.patch("uaclient.util.request.urlopen") as m_urlopen:
             util.readurl("http://some_url")
         assert 1 == m_urlopen.call_count
@@ -533,8 +761,12 @@ JSON_TEST_PAIRS = (
     # See the note in DatetimeAwareJSONDecoder for why this datetime is in a
     # dict
     (
-        {"dt": datetime.datetime(2019, 7, 25, 14, 35, 51)},
-        '{"dt": "2019-07-25T14:35:51"}',
+        {
+            "dt": datetime.datetime(
+                2019, 7, 25, 14, 35, 51, tzinfo=datetime.timezone.utc
+            )
+        },
+        '{"dt": "2019-07-25T14:35:51+00:00"}',
     ),
 )
 
@@ -588,6 +820,8 @@ class TestIsConfigValueTrue:
         [
             ({}, False),
             ({}, False),
+            (None, False),
+            ({None}, False),
             ({"allow_beta": "true"}, True),
             ({"allow_beta": "True"}, True),
             ({"allow_beta": "false"}, False),
@@ -628,3 +862,195 @@ class TestIsConfigValueTrue:
             value=key_val,
         )
         assert expected_msg == str(excinfo.value)
+
+
+class TestRedactSensitiveLogs:
+    @pytest.mark.parametrize(
+        "raw_log,expected",
+        (
+            ("Super valuable", "Super valuable"),
+            (
+                "Hi 'Bearer not the droids you are looking for', data",
+                "Hi 'Bearer <REDACTED>', data",
+            ),
+            (
+                "Hi 'Bearer not the droids you are looking for', data",
+                "Hi 'Bearer <REDACTED>', data",
+            ),
+            (
+                "Executed with sys.argv: ['/usr/bin/ua', 'attach', 'SEKRET']",
+                "Executed with sys.argv:"
+                " ['/usr/bin/ua', 'attach', '<REDACTED>']",
+            ),
+            (
+                "'resourceTokens': [{'token': 'SEKRET', 'type': 'cc-eal'}]'",
+                "'resourceTokens':"
+                " [{'token': '<REDACTED>', 'type': 'cc-eal'}]'",
+            ),
+            (
+                "'machineToken': 'SEKRET', 'machineTokenInfo': 'blah'",
+                "'machineToken': '<REDACTED>', 'machineTokenInfo': 'blah'",
+            ),
+        ),
+    )
+    def test_redact_all_matching_regexs(self, raw_log, expected):
+        """Redact all sensitive matches from log messages."""
+        assert expected == util.redact_sensitive_logs(raw_log)
+
+
+class TestParseRFC3339Date:
+    @pytest.mark.parametrize(
+        "datestring,expected",
+        [
+            (
+                "2001-02-03T04:05:06",
+                datetime.datetime(
+                    2001, 2, 3, 4, 5, 6, tzinfo=datetime.timezone.utc
+                ),
+            ),
+            (
+                "2001-02-03T04:05:06Z",
+                datetime.datetime(
+                    2001, 2, 3, 4, 5, 6, tzinfo=datetime.timezone.utc
+                ),
+            ),
+            (
+                "2001-02-03T04:05:06-08:00",
+                datetime.datetime(
+                    2001,
+                    2,
+                    3,
+                    4,
+                    5,
+                    6,
+                    tzinfo=datetime.timezone(-datetime.timedelta(hours=8)),
+                ),
+            ),
+            (
+                "2001-02-03T04:05:06+03:00",
+                datetime.datetime(
+                    2001,
+                    2,
+                    3,
+                    4,
+                    5,
+                    6,
+                    tzinfo=datetime.timezone(datetime.timedelta(hours=3)),
+                ),
+            ),
+            (
+                "2021-05-07T09:46:37.791Z",
+                datetime.datetime(
+                    2021, 5, 7, 9, 46, 37, tzinfo=datetime.timezone.utc
+                ),
+            ),
+            (
+                "2021-05-28T14:42:37.944609726-04:00",
+                datetime.datetime(
+                    2021,
+                    5,
+                    28,
+                    14,
+                    42,
+                    37,
+                    tzinfo=datetime.timezone(-datetime.timedelta(hours=4)),
+                ),
+            ),
+        ],
+    )
+    def test_parse_rfc3339_date_from_golang(self, datestring, expected):
+        """
+        Check we are able to parse dates generated by golang's MarshalJSON
+        """
+        assert expected == util.parse_rfc3339_date(datestring)
+
+
+class TestValidateProxy:
+    @pytest.mark.parametrize(
+        "proxy", ["invalidurl", "htp://wrongscheme", "http//missingcolon"]
+    )
+    @mock.patch("urllib.request.OpenerDirector.open")
+    def test_fails_on_invalid_url(self, m_open, proxy):
+        """
+        Check that invalid urls are rejected with the correct message
+        and that we don't even attempt to use them
+        """
+        with pytest.raises(exceptions.UserFacingError) as e:
+            util.validate_proxy("http", proxy, "http://example.com")
+
+        assert (
+            e.value.msg
+            == status.MESSAGE_NOT_SETTING_PROXY_INVALID_URL.format(proxy=proxy)
+        )
+
+    @pytest.mark.parametrize(
+        "protocol, proxy, test_url",
+        [
+            ("http", "http://localhost:1234", "http://example.com"),
+            ("https", "http://localhost:1234", "https://example.com"),
+            ("https", "https://localhost:1234", "https://example.com"),
+        ],
+    )
+    @mock.patch("urllib.request.Request")
+    @mock.patch("urllib.request.ProxyHandler")
+    @mock.patch("urllib.request.build_opener")
+    @mock.patch("urllib.request.OpenerDirector.open")
+    def test_calls_open_on_valid_url(
+        self,
+        m_open,
+        m_build_opener,
+        m_proxy_handler,
+        m_request,
+        protocol,
+        proxy,
+        test_url,
+    ):
+        """
+        Check that we attempt to use a valid url as a proxy
+        Also check that we return the proxy value when the open call succeeds
+        """
+        m_build_opener.return_value = urllib.request.OpenerDirector()
+        ret = util.validate_proxy(protocol, proxy, test_url)
+
+        assert [mock.call(test_url, method="HEAD")] == m_request.call_args_list
+        assert [mock.call({protocol: proxy})] == m_proxy_handler.call_args_list
+        assert 1 == m_build_opener.call_count
+        assert 1 == m_open.call_count
+
+        assert proxy == ret
+
+    @pytest.mark.parametrize(
+        "open_side_effect, expected_message",
+        [
+            (socket.timeout(0, "timeout"), "[Errno 0] timeout"),
+            (urllib.error.URLError("reason"), "reason"),
+        ],
+    )
+    @mock.patch("urllib.request.OpenerDirector.open")
+    def test_fails_when_open_fails(
+        self, m_open, open_side_effect, expected_message, caplog_text
+    ):
+        """
+        Check that we return the appropriate error when the proxy doesn't work
+        """
+        m_open.side_effect = open_side_effect
+        with pytest.raises(exceptions.UserFacingError) as e:
+            util.validate_proxy(
+                "http", "http://localhost:1234", "http://example.com"
+            )
+
+        assert (
+            e.value.msg
+            == status.MESSAGE_NOT_SETTING_PROXY_NOT_WORKING.format(
+                proxy="http://localhost:1234"
+            )
+        )
+
+        assert (
+            status.MESSAGE_ERROR_USING_PROXY.format(
+                proxy="http://localhost:1234",
+                test_url="http://example.com",
+                error=expected_message,
+            )
+            in caplog_text()
+        )
